"""
Example Voltron view.

Copy this to your ~/.voltron/plugins directory. When the `voltron view` command
is executed, 'example' should be visible in the list of valid view names.

Start your debugger as follows:

$ lldb /tmp/inferior
Voltron loaded.
Run `voltron init` after you load a target.
(lldb) target create "/tmp/inferior"
Current executable set to '/tmp/inferior' (x86_64).
(lldb) voltron init
Registered stop-hook
(lldb) b main
Breakpoint 1: where = inferior`main, address = 0x0000000100000cf0
(lldb) run
Process 13185 launched: '/Volumes/Data/Users/snare/code/voltron/repo/tests/inferior' (x86_64)
Process 13185 stopped
* thread #1: tid = 0x1ee63, 0x0000000100000cf0 inferior`main, queue = 'com.apple.main-thread', stop reason = breakpoint 1.1
    frame #0: 0x0000000100000cf0 inferior`main
inferior`main:
-> 0x100000cf0:  push   rbp
   0x100000cf1:  mov    rbp, rsp
   0x100000cf4:  sub    rsp, 0x50
   0x100000cf8:  mov    dword ptr [rbp - 0x4], 0x0

Run this view in another terminal (as follows). Each time you `stepi` in the
debugger, the view will update and display the current register values.

$ voltron view example
"""

from voltron.view import TerminalView
from voltron.plugin import ViewPlugin


class ExampleView(TerminalView):
    def render(self, *args, **kwargs):
        # Perform the request
        res = self.client.perform_request('registers')
        if res.is_success:
            # Process the registers and set the body to the formatted list
            reg_list =  ['rax','rbx','rcx','rdx','rbp','rsp','rdi','rsi','rip',
                         'r8','r9','r10','r11','r12','r13','r14','r15']
            lines = map(lambda x: '{:3}: {:016X}'.format(x, res.registers[x]), reg_list)
            self.body = '\n'.join(lines)
        else:
            self.body = "Failed to get registers: {}".format(res)

        # Set the title and info
        self.title = '[example]'
        self.info = 'some infoz'

        # Let the parent do the rendering
        super(ExampleView, self).render()


class ExampleViewPlugin(ViewPlugin):
    name = 'example'
    view_class = ExampleView



#!/usr/bin/env python
"""
Example Voltron client.

Start your debugger as follows:

$ lldb /tmp/inferior
Voltron loaded.
Run `voltron init` after you load a target.
(lldb) target create "/tmp/inferior"
Current executable set to '/tmp/inferior' (x86_64).
(lldb) voltron init
Registered stop-hook
(lldb) b main
Breakpoint 1: where = inferior`main, address = 0x0000000100000cf0
(lldb) run
Process 13185 launched: '/Volumes/Data/Users/snare/code/voltron/repo/tests/inferior' (x86_64)
Process 13185 stopped
* thread #1: tid = 0x1ee63, 0x0000000100000cf0 inferior`main, queue = 'com.apple.main-thread', stop reason = breakpoint 1.1
    frame #0: 0x0000000100000cf0 inferior`main
inferior`main:
-> 0x100000cf0:  push   rbp
   0x100000cf1:  mov    rbp, rsp
   0x100000cf4:  sub    rsp, 0x50
   0x100000cf8:  mov    dword ptr [rbp - 0x4], 0x0

Run this client in another terminal. Each time you `stepi` in the debugger,
the client will output the current RIP:

$ python client.py
Instruction pointer is: 0x100000CFF
Instruction pointer is: 0x100000D02
Instruction pointer is: 0x100000D06
Instruction pointer is: 0x100000D0D
Instruction pointer is: 0x100000D15
Instruction pointer is: 0x100000D1C
"""

import voltron
from voltron.core import Client


def main():
    # Create a client and connect to the server
    client = Client()

    # Main event loop
    while True:
        # Wait for the debugger to stop again
        res = client.perform_request('version', block=True)
        if res.is_success:
            # If nothing went wrong, get the instruction pointer and print it
            res = client.perform_request('registers', registers=['rip'])
            if res.is_success:
                print("Instruction pointer is: 0x{:X}".format(res.registers['rip']))
            else:
                print("Failed to get registers: {}".format(res))
        else:
            print("Error waiting for the debugger to stop: {}".format(res))
            break


if __name__ == "__main__":
    main()



import logging
import six
import pygments
import pygments.formatters
from pygments.token import *

from voltron.view import TerminalView, VoltronView
from voltron.plugin import ViewPlugin, api_request

log = logging.getLogger("view")


class MemoryView(TerminalView):
    printable_filter = ''.join([(len(repr(chr(x))) == 3) and chr(x) or '.' for x in range(256)])

    asynchronous = True
    last_memory = None
    last_address = 0

    @classmethod
    def configure_subparser(cls, subparsers):
        sp = subparsers.add_parser('memory', help='display a chunk of memory', aliases=('m', 'mem'))
        VoltronView.add_generic_arguments(sp)
        group = sp.add_mutually_exclusive_group(required=False)
        group.add_argument('--deref', '-d', action='store_true',
                           help='display the data in a column one CPU word wide and dereference any valid pointers',
                           default=False)
        group.add_argument('--bytes', '-b', action='store', type=int, help='bytes per line (default 16)', default=16)
        group.add_argument('--words', '-w', action='store', type=int, help='machine words per line', default=0)
        sp.add_argument('--reverse', '-v', action='store_true', help='reverse the output', default=False)
        sp.add_argument('--track', '-t', action='store_true', help='track and highlight changes', default=True)
        sp.add_argument('--no-track', '-T', action='store_false', help='don\'t track and highlight changes')
        
        group = sp.add_mutually_exclusive_group(required=False)
        group.add_argument('--address', '-a', action='store',
                           help='address (in hex or decimal) from which to start reading memory')
        group.add_argument('--command', '-c', action='store',
                           help='command to execute resulting in the address from which to start reading memory. '
                                'voltron will do his almighty best to find an address. e.g. "print \$rip + 0x1234"',
                           default=None)
        group.add_argument('--register', '-r', action='store',
                           help='register containing the address from which to start reading memory', default=None)
        sp.set_defaults(func=MemoryView)

    def build_requests(self):
        height, width = self.window_size()

        # check args
        if self.args.register:
            args = {'register': self.args.register}
        elif self.args.command:
            args = {'command': self.args.command}
        elif self.args.address:
            if self.args.address.startswith('0x'):
                addr = int(self.args.address, 16)
            else:
                try:
                    addr = int(self.args.address, 10)
                except:
                    addr = int(self.args.address, 16)
            args = {'address': addr}
        else:
            args = {'register': 'sp'}

        if self.args.deref:
            args['words'] = height
            args['offset'] = self.scroll_offset if self.args.reverse else -self.scroll_offset
        else:
            args['length'] = height * self.args.bytes * 2
            args['offset'] = self.scroll_offset * self.args.bytes * (1 if self.args.reverse else -1)

        # get memory and target info
        return [
            api_request('targets'),
            api_request('memory', deref=self.args.deref is True, **args)
        ]

    def generate_tokens(self, results):
        t_res, m_res = results

        if t_res and t_res.is_success and len(t_res.targets) > 0:
            target = t_res.targets[0]

        if m_res and m_res.is_success:
            bytes_per_chunk = self.args.words*target['addr_size'] if self.args.words else self.args.bytes
            for c in range(0, m_res.bytes, bytes_per_chunk):
                chunk = m_res.memory[c:c + bytes_per_chunk]
                yield (Name.Label, self.format_address(m_res.address + c, size=target['addr_size'], pad=False))
                yield (Name.Label, ': ')

                # Hex bytes
                byte_array = []
                for i, x in enumerate(six.iterbytes(chunk)):
                    n = "%02X" % x
                    token = Text if x else Comment
                    if self.args.track and self.last_memory and self.last_address == m_res.address:
                        if x != six.indexbytes(self.last_memory, c + i):
                            token = Error
                    byte_array.append((token, n))

                if self.args.words:
                    if target['byte_order']  =='little':
                        byte_array_words = [byte_array[i:i+ target['addr_size']] for i in range(0, bytes_per_chunk, target['addr_size'])]
                        for word in byte_array_words:
                            word.reverse()
                            for x in word:
                                yield x
                            yield (Text, ' ')
                else:
                    for x in byte_array:
                        yield x
                        yield (Text, ' ')

                # ASCII representation
                yield (Punctuation, '| ')
                for i, x in enumerate(six.iterbytes(chunk)):
                    token = String.Char if x else Comment
                    if self.args.track and self.last_memory and self.last_address == m_res.address:
                        if x != six.indexbytes(self.last_memory, c + i):
                            token = Error
                    yield (token, ((x <= 127 and self.printable_filter[x]) or '.'))
                yield (Punctuation, ' | ')

                # Deref chain
                if self.args.deref:
                    chain = m_res.deref.pop(0)
                    for i, (t, item) in enumerate(chain):
                        if t == "pointer":
                            yield (Number.Hex, self.format_address(item, size=target['addr_size'], pad=False))
                        elif t == "string":
                            for r in ['\n', '\r', '\v']:
                                item = item.replace(r, '\\{:x}'.format(ord(r)))
                            yield (String.Double, '"' + item + '"')
                        elif t == "unicode":
                            for r in ['\n', '\r', '\v']:
                                item = item.replace(r, '\\{:x}'.format(ord(r)))
                            yield (String.Double, 'u"' + item + '"')
                        elif t == "symbol":
                            yield (Name.Function, '`' + item + '`')
                        elif t == "circular":
                            yield (Text, '(circular)')
                        if i < len(chain) - 1:
                            yield (Punctuation, ' => ')

                yield (Text, '\n')

    def render(self, results):
        target = None
        self.trunc_top = self.args.reverse

        t_res, m_res = results

        if t_res and t_res.is_success and len(t_res.targets) > 0:
            target = t_res.targets[0]

            if self.args.deref or self.args.words:
                self.args.bytes = target['addr_size']

            f = pygments.formatters.get_formatter_by_name(self.config.format.pygments_formatter,
                                                          style=self.config.format.pygments_style)

            if m_res and m_res.is_success:
                lines = pygments.format(self.generate_tokens(results), f).split('\n')
                self.body = '\n'.join(reversed(lines)).strip() if self.args.reverse else '\n'.join(lines)
                self.info = '[0x{0:0=4x}:'.format(len(m_res.memory)) + self.config.format.addr_format.format(m_res.address) + ']'
            else:
                log.error("Error reading memory: {}".format(m_res.message))
                self.body = pygments.format([(Error, m_res.message)], f)
                self.info = ''

            # Store the memory
            if self.args.track:
                self.last_address = m_res.address
                self.last_memory = m_res.memory
        else:
            self.body = self.colour("Failed to get targets", 'red')

        if not self.title:
            self.title = "[memory]"

        super(MemoryView, self).render(results)

    def format_address(self, address, size=8, pad=True, prefix='0x'):
        fmt = '{:' + ('0=' + str(size * 2) if pad else '') + 'X}'
        addr_str = fmt.format(address)
        if prefix:
            addr_str = prefix + addr_str
        return addr_str


class MemoryViewPlugin(ViewPlugin):
    plugin_type = 'view'
    name = 'memory'
    view_class = MemoryView


class StackView(MemoryView):
    @classmethod
    def configure_subparser(cls, subparsers):
        sp = subparsers.add_parser('stack', help='display a chunk of stack memory', aliases=('s', 'st'))
        VoltronView.add_generic_arguments(sp)
        sp.set_defaults(func=StackView)
        sp.add_argument('--reverse', '-v', action='store_false', help='(un)reverse the output', default=True)
        sp.add_argument('--track', '-t', action='store_true', help='track and highlight changes', default=True)
        sp.add_argument('--no-track', '-T', action='store_false', help='don\'t track and highlight changes')

    def build_requests(self):
        self.args.deref = True
        self.args.words = False
        self.args.register = 'sp'
        self.args.command = None
        self.args.address = None
        self.args.bytes = None

        return super(StackView, self).build_requests()

    def render(self, results):
        self.title = '[stack]'

        super(StackView, self).render(results)


class StackViewPlugin(ViewPlugin):
    plugin_type = 'view'
    name = 'stack'
    view_class = StackView



import voltron
import voltron.api
from voltron.api import *

from scruffy.plugin import Plugin


class APIVersionRequest(APIRequest):
    """
    API version request.

    {
        "type":         "request",
        "request":      "version"
    }
    """
    @server_side
    def dispatch(self):
        res = APIVersionResponse()
        res.api_version = voltron.api.version
        res.host_version = voltron.debugger.version()
        res.capabilities = voltron.debugger.capabilities()
        return res


class APIVersionResponse(APISuccessResponse):
    """
    API version response.

    {
        "type":         "response",
        "status":       "success",
        "data": {
            "api_version":  1.0,
            "host_version": 'lldb-something',
            "capabilities": ["async"]
        }
    }
    """
    _fields = {
        'api_version': True,
        'host_version': True,
        'capabilities': False
    }

    api_version = None
    host_version = None
    capabilities = None


class APIVersionPlugin(APIPlugin):
    request = 'version'
    request_class = APIVersionRequest
    response_class = APIVersionResponse




try:
    # for some reason the relative import doesn't work in VSCode's
    # interactive debugger, but this does but I'm not sure if there's a
    # chance a different voltron could be somewhere in sys.path, so
    # let's try the relative import first
    from .main import main
except ImportError:
    from voltron.main import main

main()



import os
import argparse
import logging
import traceback
import logging
import logging.config

import voltron
from .view import *
from .core import *

log = logging.getLogger('main')


def main(debugger=None):
    voltron.setup_logging('main')

    # Set up command line arg parser
    parser = argparse.ArgumentParser()
    parser.register('action', 'parsers', AliasedSubParsersAction)
    parser.add_argument('--debug', '-d', action='store_true', help='print debug logging')
    parser.add_argument('-o', action='append', help='override config variable', default=[])
    top_level_sp = parser.add_subparsers(title='subcommands', description='valid subcommands', dest='subcommand')
    top_level_sp.required = True
    view_parser = top_level_sp.add_parser('view', help='display a view', aliases=('v'))
    view_parser.register('action', 'parsers', AliasedSubParsersAction)
    view_sp = view_parser.add_subparsers(title='views', description='valid view types', help='additional help', dest='view')
    view_sp.required = True

    # Set up a subcommand for each view class
    pm = PluginManager()
    pm.register_plugins()
    for plugin in pm.view_plugins:
        pm.view_plugins[plugin].view_class.configure_subparser(view_sp)

    # Parse args
    args = parser.parse_args()
    if args.debug:
        voltron.config['general']['debug_logging'] = True
        voltron.setup_logging('main')
    voltron.config.update(options=dict((tuple(x.split('=')) for x in args.o)))

    # Instantiate and run the appropriate module
    inst = args.func(args, loaded_config=voltron.config)
    inst.pm = pm
    try:
        inst.run()
    except Exception as e:
        log.exception("Exception running module {}: {}".format(inst.__class__.__name__, traceback.format_exc()))
        print("Encountered an exception while running the view '{}':\n{}".format(inst.__class__.__name__, traceback.format_exc()))
    except KeyboardInterrupt:
        suppress_exit_log = True
    inst.cleanup()



import logging

import voltron
from voltron.api import *

from scruffy.plugin import Plugin

log = logging.getLogger('api')

class APITargetsRequest(APIRequest):
    """
    API list targets request.

    {
        "type":         "request",
        "request":      "targets"
    }
    """
    _fields = {}

    @server_side
    def dispatch(self):
        try:
            res = APITargetsResponse()
            res.targets = voltron.debugger.targets()
        except NoSuchTargetException:
            res = APINoSuchTargetErrorResponse()
        except Exception as e:
            msg = "Exception getting targets from debugger: {}".format(repr(e))
            log.exception(msg)
            res = APIGenericErrorResponse(msg)

        return res


class APITargetsResponse(APISuccessResponse):
    """
    API list targets response.

    {
        "type":         "response",
        "status":       "success",
        "data": {
            "targets": [{
                "id":       0,         # ID that can be used in other funcs
                "file":     "/bin/ls", # target's binary file
                "arch":     "x86_64",  # target's architecture
                "state:     "stopped"  # state
            }]
        }
    }
    """
    _fields = {'targets': True}

    targets = []


class APITargetsPlugin(APIPlugin):
    request = 'targets'
    request_class = APITargetsRequest
    response_class = APITargetsResponse




from __future__ import print_function

import os
import sys
import logging
import pprint
import re
import signal
import argparse
import subprocess
import socket
from blessed import Terminal

try:
    import urwid
except:
    urwid = None

try:
    import cursor
except:
    cursor = None

import voltron
from .core import Client
from .colour import fmt_esc
from .plugin import *
from .api import BlockingNotSupportedError

log = logging.getLogger("view")

ADDR_FORMAT_128 = '0x{0:0=32X}'
ADDR_FORMAT_64 = '0x{0:0=16X}'
ADDR_FORMAT_32 = '0x{0:0=8X}'
ADDR_FORMAT_16 = '0x{0:0=4X}'
SHORT_ADDR_FORMAT_128 = '{0:0=32X}'
SHORT_ADDR_FORMAT_64 = '{0:0=16X}'
SHORT_ADDR_FORMAT_32 = '{0:0=8X}'
SHORT_ADDR_FORMAT_16 = '{0:0=4X}'


# https://gist.github.com/sampsyo/471779
class AliasedSubParsersAction(argparse._SubParsersAction):
    class _AliasedPseudoAction(argparse.Action):
        def __init__(self, name, aliases, help):
            dest = name
            if aliases:
                dest += ' (%s)' % ','.join(aliases)
            sup = super(AliasedSubParsersAction._AliasedPseudoAction, self)
            sup.__init__(option_strings=[], dest=dest, help=help)

    def add_parser(self, name, **kwargs):
        aliases = kwargs.pop('aliases', [])
        parser = super(AliasedSubParsersAction, self).add_parser(name, **kwargs)

        # Make the aliases work.
        for alias in aliases:
            self._name_parser_map[alias] = parser
        # Make the help text reflect them, first removing old help entry.
        if 'help' in kwargs:
            help = kwargs.pop('help')
            self._choices_actions.pop()
            pseudo_action = self._AliasedPseudoAction(name, aliases, help)
            self._choices_actions.append(pseudo_action)

        return parser


class AnsiString(object):
    def __init__(self, string):
        chunks = string.split('\033')
        self.chars = []
        chars = list(chunks[0])

        if len(chunks) > 1:
            for chunk in chunks[1:]:
                if chunk == '(B':
                    chars.append('\033' + chunk)
                else:
                    p = chunk.find('m')
                    if p > 0:
                        chars.append('\033' + chunk[:p + 1])
                        chars.extend(list(chunk[p + 1:]))
                    else:
                        chars.extend(list(chunk))

        # roll up ansi sequences
        ansi = []
        for char in chars:
            if char[0] == '\033':
                ansi.append(char)
            else:
                self.chars.append(''.join(ansi) + char)
                ansi = []
        if len(self.chars) > 2:
            if self.chars[-1][0] == '\033':
                self.chars[-2] = self.chars[-2] + self.chars[-1]
                self.chars = self.chars[:-1]

    def __getitem__(self, key):
        if isinstance(key, slice):
            return ''.join(self.chars[key.start:key.stop]) + '\033[0m'
        else:
            return self.chars[key]

    def __str__(self):
        return ''.join(self.chars)

    def __len__(self):
        return len(self.chars)

    def clean(self):
        return re.sub('\033\[.{1,2}m', '', str(self))


def requires_async(func):
    def inner(self, *args, **kwargs):
        if not self.block:
            return func(self, *args, **kwargs)
        else:
            sys.stdout.write('\a')
            sys.stdout.flush()
    return inner


class VoltronView (object):
    """
    Parent class for all views.

    Views may or may not support blocking mode. LLDB can be queried from a background thread, which means requests can
    (if it makes sense) be fulfilled as soon as they're received. GDB cannot be queried from a background thread, so
    requests have to be queued and dispatched by the main thread when the debugger stops. This means that GDB requires
    blocking mode.

    In blocking mode, the view's `render` method must make a single call to Client's send_request/send_request method,
    with the request(s) flagged as blocking (block=True). If a view (or, more likely, a more complex client) has
    multiple calls to send_request/send_requests, then it cannot support blocking mode and should be flagged as such
    by including `supports_blocking = False` (see below, views are flagged as supporting blocking by default).

    All of the included Voltron views support blocking mode, but this distinction has been made so that views can be
    written for LLDB only without making compromises to support GDB.
    """
    view_type = None
    block = False
    supports_blocking = True

    @classmethod
    def add_generic_arguments(cls, sp):
        sp.add_argument('--show-header', '-e', dest="header", action='store_true', help='show header', default=None)
        sp.add_argument('--hide-header', '-E', dest="header", action='store_false', help='hide header')
        sp.add_argument('--show-footer', '-f', dest="footer", action='store_true', help='show footer', default=None)
        sp.add_argument('--hide-footer', '-F', dest="footer", action='store_false', help='hide footer')
        sp.add_argument('--name', '-n', action='store', help='named configuration to use', default=None)

    @classmethod
    def configure_subparser(cls, subparsers):
        if hasattr(cls._plugin, 'aliases'):
            sp = subparsers.add_parser(cls.view_type, aliases=cls._plugin.aliases, help='{} view'.format(cls.view_type))
        else:
            sp = subparsers.add_parser(cls.view_type, help='{} view'.format(cls.view_type))
        VoltronView.add_generic_arguments(sp)
        sp.set_defaults(func=cls)

    def __init__(self, args={}, loaded_config={}):
        log.debug('Loading view: ' + self.__class__.__name__)
        self.client = Client(url=voltron.config.view.api_url)
        self.pm = None
        self.args = args
        self.loaded_config = loaded_config
        self.server_version = None

        # Commonly set by render method for header and footer formatting
        self.title = ''
        self.info = ''
        self.body = ''

        # Build configuration
        self.build_config()

        log.debug("View config: " + pprint.pformat(self.config))
        log.debug("Args: " + str(self.args))

        # Let subclass do any setup it needs to do
        self.setup()

        # Override settings from command line args
        if self.args.header != None:
            self.config.header.show = self.args.header
        if self.args.footer != None:
            self.config.footer.show = self.args.footer

        # Setup a SIGWINCH handler so we do reasonable things on resize
        try:
            signal.signal(signal.SIGWINCH, self.sigwinch_handler)
        except:
            pass

    def build_config(self):
        # Start with all_views config
        self.config = self.loaded_config.view.all_views

        # Add view-specific config
        self.config.type = self.view_type
        name = self.view_type + '_view'
        if 'view' in self.loaded_config and name in self.loaded_config.view:
            self.config.update(self.loaded_config.view[name])

        # Add named config
        if self.args.name != None:
            self.config.update(self.loaded_config[self.args.name])

        # Apply view-specific command-line args
        self.apply_cli_config()

    def apply_cli_config(self):
        if self.args.header != None:
            self.config.header.show = self.args.header
        if self.args.footer != None:
            self.config.footer.show = self.args.footer

    def setup(self):
        log.debug('Base view class setup')

    def cleanup(self):
        log.debug('Base view class cleanup')

    def render(self, results):
        log.warning('Might wanna implement render() in this view eh')

    def do_render(error=None):
        pass

    def should_reconnect(self):
        try:
            return self.loaded_config.view.reconnect
        except:
            return True

    def sigwinch_handler(self, sig, stack):
        pass


class TerminalView (VoltronView):
    valid_key_funcs = ["exit", "page_up", "page_down", "page_up", "page_down",
                       "line_up", "line_down", "reset"]

    def __init__(self, *a, **kw):
        self.init_window()
        self.trunc_top = False
        self.done = False
        self.last_body = None
        self.scroll_offset = 0
        super(TerminalView, self).__init__(*a, **kw)

    def init_window(self):
        self.t = Terminal()
        print(self.t.civis)
        if cursor:
            cursor.hide()

    def cleanup(self):
        log.debug('Cleaning up view')
        print(self.t.cnorm)
        if cursor:
            cursor.show()

    def clear(self):
        # blessed's clear doesn't work properly on windaz
        # maybe figure out the right way to do it some time
        os.system('clear')

    def render(self, results):
        self.do_render()

    def do_render(self, error=None):
        # If we got an error, we'll use that as the body
        if error:
            self.body = self.colour(error, 'red')

        # Refresh the formatted body
        self.fmt_body = self.body

        # Pad and truncate the body
        self.pad_body()
        self.truncate_body()

        if self.body != self.last_body:
            # Clear the screen
            self.clear()

            # Print the header, body and footer
            try:
                if self.config.header.show:
                    print(self.format_header_footer(self.config.header))
                print(self.fmt_body, end='')
                if self.config.footer.show:
                    print('\n' + self.format_header_footer(self.config.footer), end='')
                sys.stdout.flush()
            except IOError as e:
                # if we get an EINTR while printing, just do it again
                if e.errno == socket.EINTR:
                    self.do_render()

        self.last_body = self.body

    def sigwinch_handler(self, sig, stack):
        self.do_render()

    def window_size(self):
        height, width = subprocess.check_output(['stty', 'size']).split()
        height = int(height) - int(self.config.pad.pad_bottom)
        width = int(width) - int(self.config.pad.pad_right)
        return (height, width)

    def body_height(self):
        height, width = self.window_size()
        if self.config.header.show:
            height -= 1
        if self.config.footer.show:
            height -= 1
        return height

    def colour(self, text='', colour=None, background=None, attrs=[]):
        s = ''
        if colour:
            s += fmt_esc(colour)
        if background:
            s += fmt_esc('b_' + background)
        if attrs != []:
            s += ''.join(map(lambda x: fmt_esc('a_' + x), attrs))
        s += text
        s += fmt_esc('reset')
        return s

    def format_header_footer(self, c):
        height, width = self.window_size()

        # Get values for labels
        l = getattr(self, c.label_left.name) if c.label_left.name != None else ''
        r = getattr(self, c.label_right.name) if c.label_right.name != None else ''
        p = c.pad
        llen = len(l)
        rlen = len(r)

        # Add colour
        l = self.colour(l, c.label_left.colour, c.label_left.bg_colour, c.label_left.attrs)
        r = self.colour(r, c.label_right.colour, c.label_right.bg_colour, c.label_right.attrs)
        p = self.colour(p, c.colour, c.bg_colour, c.attrs)

        # Build
        data = l + (width - llen - rlen) * p + r

        return data

    def pad_body(self):
        height, width = self.window_size()
        lines = self.fmt_body.split('\n')
        pad = self.body_height() - len(lines)
        if pad < 0:
            pad = 0
        self.fmt_body += int(pad) * '\n'

    def truncate_body(self):
        height, width = self.window_size()

        # truncate lines horizontally
        lines = []
        for line in self.fmt_body.split('\n'):
            s = AnsiString(line)
            if len(s) > width:
                line = s[:width - 1] + self.colour('>', 'red')
            lines.append(line)

        # truncate body vertically
        if len(lines) > self.body_height():
            if self.trunc_top:
                lines = lines[len(lines) - self.body_height():]
            else:
                lines = lines[:self.body_height()]

        self.fmt_body = '\n'.join(lines)

    def build_requests(self):
        """
        Build requests for this view. Concrete view subclasses must implement
        this.
        """
        return []

    def run(self):
        """
        Run the view event loop.
        """
        def render(results=[], error=None):
            if len(results) and not results[0].timed_out:
                self.render(results)
            elif error:
                self.do_render(error=error)

        # start the client
        self.client.start(self.build_requests, render)

        # handle keyboard input
        try:
            with self.t.cbreak():
                val = ''
                while not self.done:
                    val = self.t.inkey(timeout=1)
                    if val:
                        self.handle_key(val)
        except KeyboardInterrupt:
            self.exit()

    def handle_key(self, key):
        """
        Handle a keypress. Concrete subclasses can implement this method if
        custom keypresses need to be handled other than for exit and scrolling.
        """
        try:
            func = None
            if key.is_sequence:
                try:
                    func = self.config.keymap[key.name]
                except:
                    try:
                        func = self.config.keymap[key.code]
                    except:
                        func = self.config.keymap[str(key)]
            else:
                func = self.config.keymap[str(key)]

            if func in self.valid_key_funcs:
                getattr(self, func)()
        except:
            raise

    def exit(self):
        self.cleanup()
        os._exit(0)

    @requires_async
    def page_up(self):
        self.scroll_offset += self.body_height()
        self.client.update()

    @requires_async
    def page_down(self):
        self.scroll_offset -= self.body_height()
        self.client.update()

    @requires_async
    def line_up(self):
        self.scroll_offset += 1
        self.client.update()

    @requires_async
    def line_down(self):
        self.scroll_offset -= 1
        self.client.update()

    @requires_async
    def reset(self):
        self.scroll_offset = 0
        self.client.update()



import voltron
import logging
import six
import struct

from voltron.api import *

log = logging.getLogger('api')


class APIWriteMemoryRequest(APIRequest):
    """
    API write memory request.

    {
        "type":         "request",
        "request":      "write_memory",
        "data": {
            "target_id":0,
            "address":  0x12341234,
            "data":     "\xcc"
        }
    }

    `target_id` is optional. If not present, the currently selected target
    will be used.

    `address` is the address at which to start writing.

    `data` is the data to write.
    """
    _fields = {
        'target_id': False,
        'address': True,
        'value': True
    }
    _encode_fields = ['value']

    target_id = 0

    @server_side
    def dispatch(self):
        try:
            target = voltron.debugger.target(self.target_id)

            voltron.debugger.write_memory(address=int(self.address), data=self.value, target_id=int(self.target_id))

            res = APISuccessResponse()
        except TargetBusyException:
            res = APITargetBusyErrorResponse()
        except NoSuchTargetException:
            res = APINoSuchTargetErrorResponse()
        except Exception as e:
            msg = "Exception writing memory in debugger: {}".format(repr(e))
            log.exception(msg)
            res = APIGenericErrorResponse(msg)

        return res


class APIWriteMemoryPlugin(APIPlugin):
    request = 'write_memory'
    request_class = APIWriteMemoryRequest
    response_class = APISuccessResponse



from pygments.style import Style
from pygments.token import Token, Comment, Name, Keyword, Generic, Number, Operator, String, Punctuation, Error

BASE03 = '#002b36'
BASE02 = '#073642'
BASE01 = '#586e75'
BASE00 = '#657b83'
BASE0 = '#839496'
BASE1 = '#93a1a1'
BASE2 = '#eee8d5'
BASE3 = '#fdf6e3'
YELLOW = '#b58900'
ORANGE = '#cb4b16'
RED = '#dc322f'
MAGENTA = '#d33682'
VIOLET = '#6c71c4'
BLUE = '#268bd2'
CYAN = '#2aa198'
GREEN = '#859900'


class VolarizedStyle(Style):
    background_color = BASE03
    styles = {
        Keyword: GREEN,
        Keyword.Constant: ORANGE,
        Keyword.Declaration: BASE1,
        Keyword.Namespace: ORANGE,
        # Keyword.Pseudo
        Keyword.Reserved: BLUE,
        Keyword.Type: VIOLET,

        Name: BASE1,
        Name.Attribute: BASE1,
        Name.Builtin: YELLOW,
        Name.Builtin.Pseudo: YELLOW,
        Name.Class: BLUE,
        Name.Constant: ORANGE,
        Name.Decorator: BLUE,
        Name.Entity: ORANGE,
        Name.Exception: YELLOW,
        Name.Function: BLUE,
        Name.Label: BASE01,
        # Name.Namespace
        # Name.Other
        Name.Tag: BLUE,
        Name.Variable: BLUE,
        # Name.Variable.Class
        # Name.Variable.Global
        # Name.Variable.Instance

        # Literal
        # Literal.Date
        String: BASE1,
        String.Backtick: BASE01,
        String.Char: BASE1,
        String.Doc: CYAN,
        # String.Double
        String.Escape: RED,
        String.Heredoc: CYAN,
        # String.Interpol
        # String.Other
        String.Regex: RED,
        # String.Single
        # String.Symbol
        Number: CYAN,
        # Number.Float
        # Number.Hex
        # Number.Integer
        # Number.Integer.Long
        # Number.Oct

        Operator: GREEN,
        Operator.Word: GREEN,

        Punctuation: BASE00,

        Comment: BASE00,
        # Comment.Multiline
        Comment.Preproc: GREEN,
        # Comment.Single
        Comment.Special: GREEN,

        # Generic
        Generic.Deleted: CYAN,
        Generic.Emph: 'italic',
        Generic.Error: RED,
        Generic.Heading: ORANGE,
        Generic.Inserted: GREEN,
        # Generic.Output
        Generic.Prompt: RED,
        Generic.Strong: 'bold',
        Generic.Subheading: ORANGE,
        # Generic.Traceback

        Token: BASE1,
        Token.Other: ORANGE,

        Error: RED
    }



import voltron
import voltron.api
from voltron.api import *

from scruffy.plugin import Plugin


class APINullRequest(APIRequest):
    """
    API null request.

    {
        "type":         "request",
        "request":      "null"
    }
    """
    @server_side
    def dispatch(self):
        return APINullResponse()


class APINullResponse(APISuccessResponse):
    """
    API null response.

    {
        "type":         "response",
        "status":       "success"
    }
    """


class APINullPlugin(APIPlugin):
    request = 'null'
    request_class = APINullRequest
    response_class = APINullResponse



from __future__ import print_function

import os
import struct
import logging
import threading
import codecs
from collections import namedtuple
from voltron.api import NoSuchThreadException
from voltron.colour import uncolour
from voltron.plugin import voltron
from voltron.dbg import (
    lock_host,
    validate_target,
    validate_busy,
    DebuggerAdaptor,
    InvalidPointerError,
    DebuggerCommand,
    DebuggerAdaptorPlugin
)

from voltron.plugins.debugger import xrange

try:
    import lldb
    HAVE_LLDB = True
except ImportError:
    HAVE_LLDB = False

log = logging.getLogger('debugger')

MAX_DEREF = 16

if HAVE_LLDB:
    class LLDBAdaptor(DebuggerAdaptor):
        """
        The interface with an instance of LLDB
        """
        def __init__(self, host=None):
            self.listeners = []
            self.host_lock = threading.RLock()
            if host:
                log.debug("Passed a debugger host")
                self.host = host
            elif lldb.debugger:
                log.debug("lldb.debugger is valid - probably running inside LLDB")
                self.host = lldb.debugger
            else:
                log.debug("No debugger host found - creating one")
                self.host = lldb.SBDebugger.Create()
                self.host.SetAsync(False)

        @property
        def host(self):
            """
            Get the debugger host object that this adaptor talks to. Used by
            custom API plugins to talk directly to the debugger.
            """
            return self._host

        @host.setter
        def host(self, value):
            self._host = value

        def normalize_triple(self, triple):
            """
            Returns a (cpu, platform, abi) triple

            Returns None for any fields that can't be elided
            """

            s = triple.split("-")
            arch, platform, abi = s[0], s[1], '-'.join(s[2:])
            if arch == "x86_64h":
                arch = "x86_64"
            return (arch, platform, abi)

        def version(self):
            """
            Get the debugger's version.

            Returns a string containing the debugger's version
            (e.g. 'lldb-310.2.37')
            """
            return self.host.GetVersionString()

        def _target(self, target_id=0):
            """
            Return information about the specified target.

            Returns data in the following structure:
            {
                "id":       0,         # ID that can be used in other funcs
                "file":     "/bin/ls", # target's binary file
                "arch":     "x86_64",  # target's architecture
                "state:     "stopped"  # state
            }
            """
            # get target
            t = self.host.GetTargetAtIndex(target_id)

            # get target properties
            d = {}
            d["id"] = target_id
            d["state"] = self.host.StateAsCString(t.process.GetState())
            d["file"] = t.GetExecutable().fullpath
            try:
                d["arch"], _, _ = self.normalize_triple(t.triple)
            except:
                d["arch"] = None
            if d["arch"] == 'i386':
                d["arch"] = 'x86'
            d["byte_order"] = 'little' if t.byte_order == lldb.eByteOrderLittle else 'big'
            d["addr_size"] = t.addr_size

            return d

        @lock_host
        def target(self, target_id=0):
            """
            Return information about the specified target.
            """
            return self._target(target_id=target_id)

        @lock_host
        def targets(self, target_ids=None):
            """
            Return information about the debugger's current targets.

            `target_ids` is an array of target IDs (or None for all targets)

            Returns data in the following structure:
            [
                {
                    "id":       0,         # ID that can be used in other funcs
                    "file":     "/bin/ls", # target's binary file
                    "arch":     "x86_64",  # target's architecture
                    "state:     "stopped"  # state
                }
            ]
            """
            # initialise returned data
            targets = []

            # if we didn't get any target IDs, get info for all targets
            if not target_ids:
                n = self.host.GetNumTargets()
                target_ids = range(n)

            # iterate through targets
            log.debug("Getting info for {} targets".format(len(target_ids)))
            for i in target_ids:
                targets.append(self._target(i))

            return targets

        @validate_target
        @lock_host
        def state(self, target_id=0):
            """
            Get the state of a given target.

            `target_id` is a target ID (or None for the first target)
            """
            target = self.host.GetTargetAtIndex(target_id)
            state = self.host.StateAsCString(target.process.GetState())

            return state

        @validate_busy
        @validate_target
        @lock_host
        def registers(self, target_id=0, thread_id=None, registers=[]):
            """
            Get the register values for a given target/thread.

            `target_id` is a target ID (or None for the first target)
            `thread_id` is a thread ID (or None for the selected thread)
            """
            # get the target
            target = self.host.GetTargetAtIndex(target_id)
            t_info = self._target(target_id)

            # get the thread
            if not thread_id:
                thread_id = target.process.selected_thread.id
            try:
                thread = target.process.GetThreadByID(thread_id)
            except:
                raise NoSuchThreadException()

            # if we got 'sp' or 'pc' in registers, change it to whatever the right name is for the current arch
            if t_info['arch'] in self.reg_names:
                if 'pc' in registers:
                    registers.remove('pc')
                    registers.append(self.reg_names[t_info['arch']]['pc'])
                if 'sp' in registers:
                    registers.remove('sp')
                    registers.append(self.reg_names[t_info['arch']]['sp'])
            else:
                raise Exception("Unsupported architecture: {}".format(t_info['arch']))

            # get the registers
            regs = thread.GetFrameAtIndex(0).GetRegisters()

            # extract the actual register values
            objs = []
            for i in xrange(len(regs)):
                objs += regs[i]
            regs = {}
            for reg in objs:
                val = 'n/a'
                if reg.value is not None:
                    try:
                        val = reg.GetValueAsUnsigned()
                    except:
                        reg = None
                elif reg.num_children > 0:
                    try:
                        children = []
                        for i in xrange(reg.GetNumChildren()):
                            children.append(int(reg.GetChildAtIndex(i, lldb.eNoDynamicValues, True).value, 16))
                        if t_info['byte_order'] == 'big':
                            children = list(reversed(children))
                        val = int(codecs.encode(struct.pack('{}B'.format(len(children)), *children), 'hex'), 16)
                    except:
                        pass
                if registers == [] or reg.name in registers:
                    regs[reg.name] = val

            return regs

        @validate_busy
        @validate_target
        @lock_host
        def stack_pointer(self, target_id=0, thread_id=None):
            """
            Get the value of the stack pointer register.

            `target_id` is a target ID (or None for the first target)
            `thread_id` is a thread ID (or None for the selected thread)
            """
            # get registers and targets
            regs = self.registers(target_id=target_id, thread_id=thread_id)
            target = self._target(target_id=target_id)

            # get stack pointer register
            if target['arch'] in self.reg_names:
                sp_name = self.reg_names[target['arch']]['sp']
                sp = regs[sp_name]
            else:
                raise Exception("Unsupported architecture: {}".format(target['arch']))

            return (sp_name, sp)

        @validate_busy
        @validate_target
        @lock_host
        def program_counter(self, target_id=0, thread_id=None):
            """
            Get the value of the program counter register.

            `target_id` is a target ID (or None for the first target)
            `thread_id` is a thread ID (or None for the selected thread)
            """
            # get registers and targets
            regs = self.registers(target_id=target_id, thread_id=thread_id)
            target = self._target(target_id=target_id)

            # get stack pointer register
            if target['arch'] in self.reg_names:
                pc_name = self.reg_names[target['arch']]['pc']
                pc = regs[pc_name]
            else:
                raise Exception("Unsupported architecture: {}".format(target['arch']))

            return (pc_name, pc)

        @validate_busy
        @validate_target
        @lock_host
        def memory(self, address, length, target_id=0):
            """
            Get the register values for .

            `address` is the address at which to start reading
            `length` is the number of bytes to read
            `target_id` is a target ID (or None for the first target)
            """
            # get the target
            target = self.host.GetTargetAtIndex(target_id)

            # read memory
            log.debug('Reading 0x{:x} bytes of memory at 0x{:x}'.format(length, address))

            error = lldb.SBError()
            memory = target.process.ReadMemory(address, length, error)

            if not error.Success():
                raise Exception("Failed reading memory: {}".format(error.GetCString()))

            return memory

        @validate_busy
        @validate_target
        @lock_host
        def stack(self, length, target_id=0, thread_id=None):
            """
            Get the register values for .

            `length` is the number of bytes to read
            `target_id` is a target ID (or None for the first target)
            `thread_id` is a thread ID (or None for the selected thread)
            """
            # get the stack pointer
            sp_name, sp = self.stack_pointer(target_id=target_id, thread_id=thread_id)

            # read memory
            memory = self.memory(sp, length, target_id=target_id)

            return memory

        @validate_busy
        @validate_target
        @lock_host
        def disassemble(self, target_id=0, address=None, count=None):
            """
            Get a disassembly of the instructions at the given address.

            `address` is the address at which to disassemble. If None, the
            current program counter is used.
            `count` is the number of instructions to disassemble.
            """
            # make sure we have an address
            if address is None:
                pc_name, address = self.program_counter(target_id=target_id)

            # disassemble
            output = self.command('disassemble -s {} -c {}'.format(address, count))
            output = uncolour(output)

            return output

        @validate_busy
        @validate_target
        @lock_host
        def dereference(self, pointer, target_id=0):
            """
            Recursively dereference a pointer for display
            """
            t = self.host.GetTargetAtIndex(target_id)
            error = lldb.SBError()

            addr = pointer
            chain = []

            # recursively dereference
            for i in range(0, MAX_DEREF):
                ptr = t.process.ReadPointerFromMemory(addr, error)
                if error.Success():
                    if ptr in chain:
                        chain.append(('circular', 'circular'))
                        break
                    chain.append(('pointer', addr))
                    addr = ptr
                else:
                    break

            if len(chain) == 0:
                raise InvalidPointerError("0x{:X} is not a valid pointer".format(pointer))

            # get some info for the last pointer
            # first try to resolve a symbol context for the address
            p, addr = chain[-1]
            sbaddr = lldb.SBAddress(addr, t)
            ctx = t.ResolveSymbolContextForAddress(sbaddr, lldb.eSymbolContextEverything)
            if ctx.IsValid() and ctx.GetSymbol().IsValid():
                # found a symbol, store some info and we're done for this pointer
                fstart = ctx.GetSymbol().GetStartAddress().GetLoadAddress(t)
                offset = addr - fstart
                chain.append(('symbol', '{} + 0x{:X}'.format(ctx.GetSymbol().name, offset)))
                log.debug("symbol context: {}".format(str(chain[-1])))
            else:
                # no symbol context found, see if it looks like a string
                log.debug("no symbol context")
                s = t.process.ReadCStringFromMemory(addr, 256, error)
                for i in range(0, len(s)):
                    if ord(s[i]) >= 128:
                        s = s[:i]
                        break
                if len(s):
                    chain.append(('string', s))

            return chain

        @lock_host
        def command(self, command=None):
            """
            Execute a command in the debugger.

            `command` is the command string to execute.
            """
            # for some reason this doesn't work - figure it out
            if command:
                res = lldb.SBCommandReturnObject()
                ci = self.host.GetCommandInterpreter()
                ci.HandleCommand(str(command), res, False)
                if res.Succeeded():
                    output = res.GetOutput()
                    return output.strip() if output else ""
                else:
                    raise Exception(res.GetError().strip())
            else:
                raise Exception("No command specified")

        @lock_host
        def disassembly_flavor(self):
            """
            Return the disassembly flavor setting for the debugger.

            Returns 'intel' or 'att'
            """
            res = lldb.SBCommandReturnObject()
            ci = self.host.GetCommandInterpreter()
            ci.HandleCommand('settings show target.x86-disassembly-flavor', res)
            if res.Succeeded():
                output = res.GetOutput().strip()
                flavor = output.split()[-1]
                if flavor == 'default':
                    flavor = 'att'
            else:
                raise Exception(res.GetError().strip())

            return flavor

        @validate_busy
        @validate_target
        @lock_host
        def breakpoints(self, target_id=0):
            """
            Return a list of breakpoints.

            Returns data in the following structure:
            [
                {
                    "id":           1,
                    "enabled":      True,
                    "one_shot":     False,
                    "hit_count":    5,
                    "locations": [
                        {
                            "address":  0x100000cf0,
                            "name":     'main'
                        }
                    ]
                }
            ]
            """
            breakpoints = []
            t = self.host.GetTargetAtIndex(target_id)
            s = lldb.SBStream()

            for i in range(0, t.GetNumBreakpoints()):
                b = t.GetBreakpointAtIndex(i)
                locations = []

                for j in range(0, b.GetNumLocations()):
                    loc = b.GetLocationAtIndex(j)
                    s.Clear()
                    loc.GetAddress().GetDescription(s)
                    desc = s.GetData()
                    locations.append({
                        'address':  loc.GetLoadAddress(),
                        'name':     desc
                    })

                breakpoints.append({
                    'id':           b.id,
                    'enabled':      b.enabled,
                    'one_shot':     b.one_shot,
                    'hit_count':    b.GetHitCount(),
                    'locations':    locations
                })

            return breakpoints

        @validate_busy
        @validate_target
        @lock_host
        def backtrace(self, target_id=0, thread_id=None):
            """
            Return a list of stack frames.
            """
            target = self.host.GetTargetAtIndex(target_id)
            if not thread_id:
                thread_id = target.process.selected_thread.id
            try:
                thread = target.process.GetThreadByID(thread_id)
            except:
                raise NoSuchThreadException()

            frames = []
            for frame in thread:
                start_addr = frame.GetSymbol().GetStartAddress().GetFileAddress()
                offset = frame.addr.GetFileAddress() - start_addr
                ctx = frame.GetSymbolContext(lldb.eSymbolContextEverything)
                mod = ctx.GetModule()
                name = '{mod}`{symbol} + {offset}'.format(mod=os.path.basename(str(mod.file)), symbol=frame.name, offset=offset)
                frames.append({'index': frame.idx, 'addr': frame.addr.GetFileAddress(), 'name': name})

            return frames

        def capabilities(self):
            """
            Return a list of the debugger's capabilities.

            Thus far only the 'async' capability is supported. This indicates
            that the debugger host can be queried from a background thread,
            and that views can use non-blocking API requests without queueing
            requests to be dispatched next time the debugger stops.
            """
            return ["async"]

        def register_command_plugin(self, name, cls):
            """
            Register a command plugin with the LLDB adaptor.
            """
            # make sure we have a commands object
            if not voltron.commands:
                voltron.commands = namedtuple('VoltronCommands', [])

            # method invocation creator
            def create_invocation(obj):
                @staticmethod
                def invoker(debugger, command, result, env_dict):
                    obj.invoke(*command.split())
                return invoker

            # store the invocation in `voltron.commands` to pass to LLDB
            setattr(voltron.commands, name, create_invocation(cls()))

            # register the invocation as a command script handler thing
            self.host.HandleCommand("command script add -f voltron.commands.{} {}".format(name, name))

    class LLDBCommand(DebuggerCommand):
        """
        Debugger command class for LLDB
        """
        @staticmethod
        def _invoke(debugger, command, *args):
            voltron.command.handle_command(command)

        def __init__(self):
            super(LLDBCommand, self).__init__()

            self.hook_idx = None
            self.adaptor = voltron.debugger

            # install the voltron command handler
            self.adaptor.command("script import voltron")
            self.adaptor.command('command script add -f entry.invoke voltron')

            # try to register hooks automatically, as this works on new LLDB versions
            self.register_hooks(True)

        def invoke(self, debugger, command, result, dict):
            self.handle_command(command)

        def register_hooks(self, quiet=False):
            try:
                output = self.adaptor.command("target stop-hook list")
                if 'voltron' not in output:
                    output = self.adaptor.command('target stop-hook add -o \'voltron stopped\'')
                    try:
                        # hahaha this sucks
                        log.debug("Saving hook index for unregistering.")
                        self.hook_idx = int(output.split()[2][1:])
                    except Exception as e:
                        log.warning(f"Exception when saving hook index for unregistering. {e}")
                        pass
                self.registered = True
                if not quiet:
                    print("Registered stop-hook")
            except:
                if not quiet:
                    print("No targets")

        def unregister_hooks(self):
            self.adaptor.command('target stop-hook delete {}'.format(self.hook_idx if self.hook_idx else ''))
            self.registered = False

    class LLDBAdaptorPlugin(DebuggerAdaptorPlugin):
        host = 'lldb'
        adaptor_class = LLDBAdaptor
        command_class = LLDBCommand



from __future__ import print_function

import logging
import threading
import array

from voltron.api import *
from voltron.plugin import *
from voltron.dbg import *

try:
    in_windbg = False
    import pykd
    try:
        import vtrace
    except:
        in_windbg = True
except ImportError:
    pass

log = logging.getLogger('debugger')


if in_windbg:
    class WinDbgAdaptor(DebuggerAdaptor):
        sizes = {
            'x86': 4,
            'x86_64': 8,
        }
        max_deref = 24
        max_string = 128

        def __init__(self, *args, **kwargs):
            self.listeners = []
            self.host_lock = threading.RLock()
            self.host = pykd

        def version(self):
            """
            Get the debugger's version.

            Returns a string containing the debugger's version
            (e.g. 'Microsoft (R) Windows Debugger Version whatever, pykd 0.3.0.38')
            """
            try:
                [windbg] = [line for line in pykd.dbgCommand('version').split('\n') if 'Microsoft (R) Windows Debugger Version' in line]
            except:
                windbg = 'WinDbg <unknown>'
            return '{}, {}'.format(windbg, 'pykd {}'.format(pykd.version))

        def _target(self, target_id=0):
            """
            Return information about the specified target.

            Returns data in the following structure:
            {
                "id":       0,         # ID that can be used in other funcs
                "file":     "/bin/ls", # target's binary file
                "arch":     "x86_64",  # target's architecture
                "state:     "stopped"  # state
            }
            """
            # get target properties
            d = {}
            d["id"] = pykd.getCurrentProcessId()
            d["num"] = d['id']

            # get target state
            d["state"] = self._state()

            d["file"] = pykd.getProcessExeName()

            # get arch
            d["arch"] = self.get_arch()
            d['byte_order'] = self.get_byte_order()
            d['addr_size'] = self.get_addr_size()

            return d

        @lock_host
        def target(self, target_id=0):
            """
            Return information about the current inferior.

            We only support querying the current inferior with WinDbg.

            `target_id` is ignored.
            """
            return self._target()

        @lock_host
        def targets(self, target_ids=None):
            """
            Return information about the debugger's current targets.

            `target_ids` is ignored. Only the current target is returned. This
            method is only implemented to maintain API compatibility with the
            LLDBAdaptor.
            """
            return [self._target()]

        @validate_target
        @lock_host
        def state(self, target_id=0):
            """
            Get the state of a given target.
            """
            return self._state()

        @validate_busy
        @validate_target
        @lock_host
        def registers(self, target_id=0, thread_id=None, registers=[]):
            """
            Get the register values for a given target/thread.
            """
            arch = self.get_arch()

            # if we got 'sp' or 'pc' in registers, change it to whatever the right name is for the current arch
            if arch in self.reg_names:
                if 'pc' in registers:
                    registers.remove('pc')
                    registers.append(self.reg_names[arch]['pc'])
                if 'sp' in registers:
                    registers.remove('sp')
                    registers.append(self.reg_names[arch]['sp'])
            else:
                raise Exception("Unsupported architecture: {}".format(target['arch']))

            # get registers
            if registers != []:
                vals = {}
                for reg in registers:
                    vals[reg] = pykd.reg(reg)
            else:
                log.debug('Getting registers for arch {}'.format(arch))
                if arch == "x86_64":
                    reg_names = ['rax', 'rbx', 'rcx', 'rdx', 'rbp', 'rsp', 'rdi', 'rsi', 'rip', 'r8', 'r9', 'r10',
                                 'r11', 'r12', 'r13', 'r14', 'r15', 'cs', 'ds', 'es', 'fs', 'gs', 'ss']
                elif arch == "x86":
                    reg_names = ['eax', 'ebx', 'ecx', 'edx', 'ebp', 'esp', 'edi', 'esi', 'eip', 'cs', 'ds', 'es',
                                 'fs', 'gs', 'ss']
                else:
                    raise UnknownArchitectureException()

                vals = {}
                for reg in reg_names:
                    try:
                        vals[reg] = pykd.reg(reg)
                    except:
                        log.debug('Failed getting reg: ' + reg)
                        vals[reg] = 'N/A'

                # Get flags
                try:
                    vals['rflags'] = pykd.reg(reg)
                except:
                    log.debug('Failed getting reg: eflags')
                    vals['rflags'] = 'N/A'

                # Get SSE registers
                try:
                    vals.update(self.get_registers_sse(16))
                except:
                    log.exception("Failed to get SSE registers")

                # Get FPU registers
                try:
                    vals.update(self.get_registers_fpu())
                except:
                    log.exception("Failed to get FPU registers")

            return vals

        @validate_busy
        @validate_target
        @lock_host
        def stack_pointer(self, target_id=0, thread_id=None):
            """
            Get the value of the stack pointer register.
            """
            arch = self.get_arch()
            if arch in self.reg_names:
                sp_name = self.reg_names[arch]['sp']
                sp = pykd.reg(sp_name)
            else:
                raise UnknownArchitectureException()

            return sp_name, sp

        @validate_busy
        @validate_target
        @lock_host
        def program_counter(self, target_id=0, thread_id=None):
            """
            Get the value of the program counter register.
            """
            arch = self.get_arch()
            if arch in self.reg_names:
                pc_name = self.reg_names[arch]['pc']
                pc = pykd.reg(pc_name)
            else:
                raise UnknownArchitectureException()

            return pc_name, pc

        @validate_busy
        @validate_target
        @lock_host
        def memory(self, address, length, target_id=0):
            """
            Get the register values for .

            `address` is the address at which to start reading
            `length` is the number of bytes to read
            """
            # read memory
            log.debug('Reading 0x{:x} bytes of memory at 0x{:x}'.format(length, address))
            memory = array.array('B', pykd.loadBytes(address, length)).tostring()

            return memory

        @validate_busy
        @validate_target
        @lock_host
        def stack(self, length, target_id=0, thread_id=None):
            """
            Get the register values for .

            `length` is the number of bytes to read
            `target_id` is a target ID (or None for the first target)
            `thread_id` is a thread ID (or None for the selected thread)
            """
            # get the stack pointer
            sp_name, sp = self.stack_pointer(target_id=target_id, thread_id=thread_id)

            # read memory
            memory = self.memory(sp, length, target_id=target_id)

            return memory

        @validate_busy
        @validate_target
        @lock_host
        def disassemble(self, target_id=0, address=None, count=16):
            """
            Get a disassembly of the instructions at the given address.

            `address` is the address at which to disassemble. If None, the
            current program counter is used.
            `count` is the number of instructions to disassemble.
            """
            # make sure we have an address
            if address is None:
                pc_name, address = self.program_counter(target_id=target_id)

            # disassemble
            output = pykd.dbgCommand('u 0x{:x} l{}'.format(address, count))

            return output

        @validate_busy
        @validate_target
        @lock_host
        def dereference(self, pointer, target_id=0):
            """
            Recursively dereference a pointer for display
            """
            fmt = ('<' if self.get_byte_order() == 'little' else '>') + {2: 'H', 4: 'L', 8: 'Q'}[self.get_addr_size()]

            addr = pointer
            chain = []

            # recursively dereference
            for i in range(0, self.max_deref):
                try:
                    [ptr] = pykd.loadPtrs(addr, 1)
                    if ptr in chain:
                        break
                    chain.append(('pointer', addr))
                    addr = ptr
                except:
                    log.exception("Dereferencing pointer 0x{:X}".format(addr))
                    break

            # get some info for the last pointer
            # first try to resolve a symbol context for the address
            if len(chain):
                p, addr = chain[-1]
                output = pykd.findSymbol(addr)
                sym = True
                try:
                    # if there's no symbol found, pykd returns a hex string of the address
                    if int(output, 16) == addr:
                        sym = False
                        log.debug("no symbol context")
                except:
                    pass

                if sym:
                    chain.append(('symbol', output.strip()))
                else:
                    log.debug("no symbol context")
                    mem = pykd.loadBytes(addr, 2)
                    if mem[0] < 127:
                        if mem[1] == 0:
                            a = []
                            for i in range(0, self.max_string, 2):
                                mem = pykd.loadBytes(addr + i, 2)
                                if mem == [0, 0]:
                                    break
                                a.extend(mem)
                            output = array.array('B', a).tostring().decode('UTF-16').encode('latin1')
                            chain.append(('unicode', output))
                        else:
                            output = pykd.loadCStr(addr)
                            chain.append(('string', output))

            log.debug("chain: {}".format(chain))
            return chain

        @lock_host
        def command(self, command=None):
            """
            Execute a command in the debugger.

            `command` is the command string to execute.
            """
            if command:
                res = pykd.dbgCommand(command)
            else:
                raise Exception("No command specified")

            return res

        @lock_host
        def disassembly_flavor(self):
            """
            Return the disassembly flavor setting for the debugger.

            Returns 'intel' or 'att'
            """
            return 'intel'

        @lock_host
        def breakpoints(self, target_id=0):
            """
            Return a list of breakpoints.

            Returns data in the following structure:
            [
                {
                    "id":           1,
                    "enabled":      True,
                    "one_shot":     False,
                    "hit_count":    5,
                    "locations": [
                        {
                            "address":  0x100000cf0,
                            "name":     'main'
                        }
                    ]
                }
            ]
            """
            breakpoints = []

            for i in range(0, pykd.getNumberBreakpoints()):
                b = pykd.getBp(i)
                addr = b.getOffset()

                name = hex(addr)
                try:
                    name = pykd.findSymbol(addr)
                except:
                    log.exception("No symbol found for address {}".format(addr))
                    pass

                breakpoints.append({
                    'id':           i,
                    'enabled':      True,
                    'one_shot':     False,
                    'hit_count':    '-',
                    'locations':    [{
                        "address":  addr,
                        "name":     name
                    }]
                })

            return breakpoints

        def capabilities(self):
            """
            Return a list of the debugger's capabilities.

            Thus far only the 'async' capability is supported. This indicates
            that the debugger host can be queried from a background thread,
            and that views can use non-blocking API requests without queueing
            requests to be dispatched next time the debugger stops.
            """
            return ['async']

        #
        # Private functions
        #

        def _state(self):
            """
            Get the state of a given target. Internal use.
            """
            s = pykd.getExecutionStatus()
            if s == pykd.executionStatus.Break:
                state = 'stopped'
            elif s == pykd.executionStatus.Go:
                state = 'running'
            else:
                state = 'invalid'

            return state

        def get_registers_sse(self, num=8):
            regs = {}
            for i in range(0, 16):
                try:
                    reg = 'xmm{}'.format(i)
                    regs[reg] = pykd.reg(reg)
                except:
                    break
            return regs

        def get_registers_fpu(self):
            regs = {}
            for i in range(0, 8):
                try:
                    reg = 'st{}'.format(i)
                    regs[reg] = pykd.reg(reg)
                except:
                    break
            return regs

        def get_next_instruction(self):
            return str(pykd.disasm())

        def get_arch(self):
            t = pykd.getCPUType()
            if t == pykd.CPUType.I386:
                return 'x86'
            else:
                return 'x86_64'
            return arch

        def get_addr_size(self):
            arch = self.get_arch()
            return self.sizes[arch]

        def get_byte_order(self):
            return 'little'


    class EventHandler(pykd.eventHandler):
        """
        Event handler for WinDbg/PyKD events.
        """
        def __init__(self, adaptor, *args, **kwargs):
            super(EventHandler, self).__init__(*args, **kwargs)
            self.adaptor = adaptor

        def onExecutionStatusChange(self, status):
            if status == pykd.executionStatus.Break:
                self.adaptor.update_state()
                voltron.server.dispatch_queue()


    class WinDbgCommand(DebuggerCommand):
        """
        Debugger command class for WinDbg
        """
        def __init__(self):
            super(WinDbgCommand, self).__init__()
            self.register_hooks()

        def invoke(self, debugger, command, result, dict):
            self.handle_command(command)

        def register_hooks(self):
            self.handler = EventHandler(self.adaptor)

        def unregister_hooks(self):
            del self.handler
            self.handler = None


    class WinDbgAdaptorPlugin(DebuggerAdaptorPlugin):
        host = 'windbg'
        adaptor_class = WinDbgAdaptor
        command_class = WinDbgCommand



"""
Example command plugin

Copy this to your ~/.voltron/plugins directory. It will be loaded when
Voltron is initialised, and register a debugger command that works as follows:

$ lldb /tmp/inferior
Voltron loaded.
Run `voltron init` after you load a target.
(lldb) target create "/tmp/inferior"
Current executable set to '/tmp/inferior' (x86_64).
(lldb) b main
Breakpoint 1: where = inferior`main, address = 0x0000000100000cf0
(lldb) run
Process 12561 launched: '/tmp/inferior' (x86_64)
Process 12561 stopped
* thread #1: tid = 0x1d33f, 0x0000000100000cf0 inferior`main, queue = 'com.apple.main-thread', stop reason = breakpoint 1.1
    frame #0: 0x0000000100000cf0 inferior`main
inferior`main:
-> 0x100000cf0:  push   rbp
   0x100000cf1:  mov    rbp, rsp
   0x100000cf4:  sub    rsp, 0x50
   0x100000cf8:  mov    dword ptr [rbp - 0x4], 0x0
(lldb) example
rax 0000000100000CF0
rbx 0000000000000000
rcx 00007FFF5FBFFA70
rdx 00007FFF5FBFF978
rbp 00007FFF5FBFF958
rsp 00007FFF5FBFF948
rdi 0000000000000001
rsi 00007FFF5FBFF968
rip 0000000100000CF0
r8  0000000000000000
r9  00007FFF5FBFEA08
r10 0000000000000032
r11 0000000000000246
r12 0000000000000000
r13 0000000000000000
r14 0000000000000000
r15 0000000000000000

Provided you stick to the adaptor API that is implemented in every *DBAdaptor
class, custom command plugins should work across all debugger hosts.

This is a quick example that will only work on an x86_64 target.
"""

import blessed
import voltron
from voltron.plugin import CommandPlugin
from voltron.command import VoltronCommand


class ExampleCommand(VoltronCommand):
    def invoke(self, *args):
        regs = voltron.debugger.registers()
        reg_list =  ['rax','rbx','rcx','rdx','rbp','rsp','rdi','rsi','rip',
                     'r8','r9','r10','r11','r12','r13','r14','r15']
        for name in reg_list:
            print("{t.bold}{:3} {t.normal}{:0=16X}".format(name, regs[name], t=blessed.Terminal()))


class ExampleCommandPlugin(CommandPlugin):
    name = 'example'
    command_class = ExampleCommand


import re
ESCAPES = {
    # reset
    'reset':        0,

    # colours
    'grey':         30,
    'red':          31,
    'green':        32,
    'yellow':       33,
    'blue':         34,
    'magenta':      35,
    'cyan':         36,
    'white':        37,

    # background
    'b_grey':       40,
    'b_red':        41,
    'b_green':      42,
    'b_yellow':     43,
    'b_blue':       44,
    'b_magenta':    45,
    'b_cyan':       46,
    'b_white':      47,

    # attributes
    'a_bold':       1,
    'a_dark':       2,
    'a_underline':  4,
    'a_blink':      5,
    'a_reverse':    7,
    'a_concealed':  8
}
ESC_TEMPLATE = '\033[{}m'

def escapes():
    return ESCAPES

def get_esc(name):
    return ESCAPES[name]

def fmt_esc(name):
    return ESC_TEMPLATE.format(escapes()[name])

FMT_ESCAPES = dict((k, fmt_esc(k)) for k in ESCAPES)


def uncolour(text):
    """
    Remove ANSI color/style sequences from a string. The set of all possible
    ANSI sequences is large, so does not try to strip every possible one. But
    does strip some outliers by other ANSI colorizers in the wild. Those
    include `\x1b[K` (aka EL or erase to end of line) and `\x1b[m`, a terse
    version of the more common `\x1b[0m`.

    Stolen from: https://github.com/jonathaneunice/colors/blob/master/colors/colors.py
    """
    text = re.sub('\x1b\\[(K|.*?m)', '', text)
    return text



"""
These tests load an inferior into an LLDB instance and then issue API requests
using the client.

Tests:
Client -> Server -> LLDBAdaptor

Using an instantiated SBDebugger instance
"""

import tempfile
import sys
import json
import time
import logging
import subprocess
import time

from mock import Mock
from nose.tools import *

import voltron
from voltron.core import *
from voltron.api import *
from voltron.plugin import *

import platform
if platform.system() == 'Darwin':
    sys.path.append("/Applications/Xcode.app/Contents/SharedFrameworks/LLDB.framework/Resources/Python")

try:
    import lldb

    from .common import *

    log = logging.getLogger("tests")

    def setup():
        global server, client, target, pm, adaptor, methods

        log.info("setting up API tests")

        # set up voltron
        voltron.setup_env()
        pm = PluginManager()
        plugin = pm.debugger_plugin_for_host('lldb')
        adaptor = plugin.adaptor_class()
        voltron.debugger = adaptor

        # start up a voltron server
        server = Server()
        # import pdb;pdb.set_trace()
        server.start()

        time.sleep(0.1)

        # set up client
        client = Client()

        # compile and load the test inferior
        subprocess.call("cc -o tests/inferior tests/inferior.c", shell=True)
        target = adaptor.host.CreateTargetWithFileAndArch("tests/inferior", lldb.LLDB_ARCH_DEFAULT)
        main_bp = target.BreakpointCreateByName("main", target.GetExecutable().GetFilename())

    def teardown():
        server.stop()
        time.sleep(5)

    def test_state_no_target():
        req = api_request('state')
        res = client.send_request(req)
        assert res.is_error
        assert res.code == 4101

    def test_state_stopped():
        process = target.LaunchSimple(None, None, os.getcwd())
        req = api_request('state')
        res = client.send_request(req)
        assert res.status == 'success'
        assert res.state == "stopped"
        target.process.Destroy()

    def test_targets():
        req = api_request('targets')
        res = client.send_request(req)
        assert res.status == 'success'
        t = res.targets[0]
        assert t["id"] == 0
        assert t["arch"] == "x86_64"
        assert t["file"].endswith("inferior")

    def test_registers():
        process = target.LaunchSimple(None, None, os.getcwd())
        req = api_request('registers')
        res = client.send_request(req)
        assert res.status == 'success'
        assert len(res.registers) > 0
        assert res.registers['rip'] != 0
        target.process.Destroy()

    def test_memory():
        process = target.LaunchSimple(None, None, os.getcwd())
        res = client.perform_request('registers')
        rsp = res.registers['rsp']
        res = client.perform_request('memory', address=rsp, length=0x40)
        assert res.status == 'success'
        assert len(res.memory) > 0
        res = client.perform_request('memory', address=rsp, length=0x40, deref=True)
        assert res.status == 'success'
        assert len(res.deref) > 0
        target.process.Destroy()

    def test_stack():
        process = target.LaunchSimple(None, None, os.getcwd())
        req = api_request('stack', length=0x40)
        res = client.send_request(req)
        assert res.status == 'success'
        assert len(res.memory) > 0
        target.process.Destroy()

    def test_command():
        process = target.LaunchSimple(None, None, os.getcwd())
        req = api_request('command', command="reg read")
        res = client.send_request(req)
        assert res.status == 'success'
        assert len(res.output) > 0
        assert 'rax' in res.output
        target.process.Destroy()

    def test_disassemble():
        process = target.LaunchSimple(None, None, os.getcwd())
        req = api_request('disassemble', count=16)
        res = client.send_request(req)
        assert res.status == 'success'
        assert len(res.disassembly) > 0
        assert 'push' in res.disassembly
        req = api_request('disassemble', count=16, use_capstone=True)
        res = client.send_request(req)
        assert res.status == 'success'
        assert len(res.disassembly) > 0
        assert 'push' in res.disassembly
        target.process.Destroy()

    def test_dereference():
        process = target.LaunchSimple(None, None, os.getcwd())
        res = client.perform_request('registers')
        res = client.perform_request('dereference', pointer=res.registers['rsp'])
        assert res.status == 'success'
        assert res.output[0][0] == 'pointer'
        assert res.output[-1][1] == 'start + 0x1'
        target.process.Destroy()

    def test_breakpoints():
        process = target.LaunchSimple(None, None, os.getcwd())
        res = client.perform_request('breakpoints')
        assert res.status == 'success'
        assert len(res.breakpoints) == 1
        assert res.breakpoints[0]['one_shot'] == False
        assert res.breakpoints[0]['enabled']
        assert res.breakpoints[0]['id'] == 1
        assert res.breakpoints[0]['hit_count'] > 0
        assert res.breakpoints[0]['locations'][0]['name'] == "inferior`main"
        target.process.Destroy()

    def test_multi_request():
        process = target.LaunchSimple(None, None, os.getcwd())
        reg_res, dis_res = client.send_requests(api_request('registers'),
                                                api_request('disassemble', count=16))
        assert reg_res.status == 'success'
        assert len(reg_res.registers) > 0
        assert reg_res.registers['rip'] != 0
        assert dis_res.status == 'success'
        assert len(dis_res.disassembly) > 0
        assert 'push' in dis_res.disassembly
        target.process.Destroy()

except:
    print("No LLDB")



import logging
from mock import Mock

LOGGER_DEFAULT = {
    'handlers': ['file'],
    'level': 'DEBUG',
    'propagate': False
}

LOG_CONFIG = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'standard': {'format': '[%(levelname)s] %(message)s'},
        'testing': {'format': "%(levelname)-7s %(filename)12s:%(lineno)-4s %(funcName)20s -- %(message)s"}
    },
    'handlers': {
        'default': {
            'class': 'logging.StreamHandler',
            'formatter': 'standard'
        },
        'file': {
            'class': 'logging.FileHandler',
            'formatter': 'testing',
            'filename': 'tests/test.log',
            'delay': True
        }
    },
    'loggers': {
        '':         LOGGER_DEFAULT,
        'debugger': LOGGER_DEFAULT,
        'core':     LOGGER_DEFAULT,
        'main':     LOGGER_DEFAULT,
        'api':      LOGGER_DEFAULT,
        'view':     LOGGER_DEFAULT,
        'plugin':   LOGGER_DEFAULT,
    }
}

logging.config.dictConfig(LOG_CONFIG)

state_response = "stopped"
targets_response = [{
    "id":       0,
    "file":     "/bin/ls",
    "arch":     "x86_64",
    "state":     "stopped"
}]
registers_response = (
    {"gs": 0, "fooff": 0, "edi": 1, "edx": 1349115624, "r13w": 0, "r8l": 0, "fiseg": 0, "r8d": 0,
        "r13d": 0, "r13l": 0, "fstat": 0, "r8w": 0, "ymm9": "n/a", "ymm8": "n/a", "r14": 0, "r15": 0, "r12": 0, "r13": 0,
        "dh": 222, "di": 1, "ymm1": "n/a", "ymm0": "n/a", "ymm3": "n/a", "ymm2": "n/a", "ymm5": "n/a", "ymm4": "n/a",
        "ymm7": "n/a", "ymm6": "n/a", "dx": 57064, "dil": 1, "xmm6": "n/a", "r10l": 0, "bpl": 200, "r10d": 1349110784,
        "xmm10": "n/a", "xmm11": "n/a", "xmm12": "n/a", "xmm13": "n/a", "xmm14": "n/a", "xmm15": "n/a", "fioff": 0,
        "sil": 216, "r10w": 52224, "mxcsr": 8064, "ebp": 1349115592, "ebx": 0, "r15d": 0, "fop": 0, "esp": 1349115576,
        "r15l": 0, "r15w": 0, "ftag": 0, "esi": 1349115608, "bl": 0, "bh": 0, "xmm2": "n/a", "xmm3": "n/a", "xmm0": "n/a",
        "xmm1": "n/a", "bp": 57032, "xmm7": "n/a", "xmm4": "n/a", "xmm5": "n/a", "xmm8": "n/a", "xmm9": "n/a", "bx": 0,
        "ecx": 1349115632, "r9l": 0, "dl": 232, "r12w": 0, "r9d": 1349111808, "r8": 0, "rdx": 140734542503656, "r12d": 0,
        "r9w": 53248, "rdi": 1, "r12l": 0, "ch": 222, "cl": 240, "stmm4": "n/a", "stmm5": "n/a", "stmm6": "n/a", "stmm7":
        "n/a", "stmm0": "n/a", "stmm1": "n/a", "stmm2": "n/a", "stmm3": "n/a", "cx": 57072, "cs": 43,
        "rcx": 140734542503664, "rflags": 582, "rsi": 140734542503640, "mxcsrmask": 65535, "eax": 257305888,
        "rsp": 140734542503608, "trapno": 3, "r14d": 0, "faultvaddr": 4552486912, "err": 0, "rbx": 0, "r14l": 0,
        "rbp": 140734542503624, "r14w": 0, "ah": 45, "al": 32, "rip": 4552273184, "r9": 140734542499840, "spl": 184,
        "ax": 11552, "fctrl": 895, "rax": 4552273184, "r11l": 70, "r10": 140734542498816, "r11": 582, "r11d": 582,
        "foseg": 0, "r11w": 582, "fs": 0, "ymm11": "n/a", "ymm10": "n/a", "ymm13": "n/a", "ymm12": "n/a", "ymm15": "n/a",
        "ymm14": "n/a", "sp": 57016, "si": 57048})
memory_response = "\xff" * 0x40
stack_response = "\xff" * 0x40
wait_response = "stopped"
command_response = "inferior`main:\n-> 0x100000d20:  pushq  %rbp\n   0x100000d21:  movq   %rsp, %rbp\n   0x100000d24:  subq   $0x40, %rsp\n   0x100000d28:  movl   $0x0, -0x4(%rbp)\n   0x100000d2f:  movl   %edi, -0x8(%rbp)\n   0x100000d32:  movq   %rsi, -0x10(%rbp)\n   0x100000d36:  movl   $0x0, -0x14(%rbp)\n   0x100000d3d:  movq   $0x0, -0x20(%rbp)\n   0x100000d45:  cmpl   $0x1, -0x8(%rbp)\n   0x100000d4c:  jle    0x100000d94               ; main + 116\n   0x100000d52:  movq   -0x10(%rbp), %rax\n   0x100000d56:  movq   0x8(%rax), %rdi\n   0x100000d5a:  leaq   0x18a(%rip), %rsi         ; \"sleep\"\n   0x100000d61:  callq  0x100000ea0               ; symbol stub for: strcmp\n   0x100000d66:  cmpl   $0x0, %eax\n   0x100000d6b:  jne    0x100000d94               ; main + 116\n   0x100000d71:  leaq   0x179(%rip), %rdi         ; \"*** Sleeping for 5 seconds\\n\"\n   0x100000d78:  movb   $0x0, %al\n   0x100000d7a:  callq  0x100000e94               ; symbol stub for: printf\n   0x100000d7f:  movl   $0x5, %edi\n   0x100000d84:  movl   %eax, -0x24(%rbp)\n   0x100000d87:  callq  0x100000e9a               ; symbol stub for: sleep\n   0x100000d8c:  movl   %eax, -0x28(%rbp)\n   0x100000d8f:  jmpq   0x100000e88               ; main + 360\n   0x100000d94:  cmpl   $0x1, -0x8(%rbp)\n   0x100000d9b:  jle    0x100000dd6               ; main + 182\n   0x100000da1:  movq   -0x10(%rbp), %rax\n   0x100000da5:  movq   0x8(%rax), %rdi\n   0x100000da9:  leaq   0x15d(%rip), %rsi         ; \"loop\"\n   0x100000db0:  callq  0x100000ea0               ; symbol stub for: strcmp\n   0x100000db5:  cmpl   $0x0, %eax\n   0x100000dba:  jne    0x100000dd6               ; main + 182"
disassemble_response = command_response
dereference_response = [[u'pointer', 140734748778168], [u'pointer', 140735677462013], [u'symbol', u'start + 0x1']]
breakpoints_response = {"status": "success", "data": {"breakpoints": [{"one_shot": False, "enabled": True, "id": 1, "hit_count": 1, "locations": [{"name": "inferior`main", "address": 4294970608}]}]}, "type": "response"}


def inject_mock(adaptor):
    adaptor.version = Mock(return_value='lldb-something')
    adaptor.state = Mock(return_value=state_response)
    adaptor.target = Mock(return_value=targets_response[0])
    adaptor._target = Mock(return_value=targets_response[0])
    adaptor.targets = Mock(return_value=targets_response)
    adaptor.registers = Mock(return_value=registers_response)
    adaptor.memory = Mock(return_value=memory_response)
    adaptor.stack = Mock(return_value=stack_response)
    adaptor.wait = Mock(return_value=wait_response)
    adaptor.command = Mock(return_value=command_response)
    adaptor.disassemble = Mock(return_value=disassemble_response)
    adaptor.dereference = Mock(return_value=dereference_response)
    adaptor.breakpoints = Mock(return_value=breakpoints_response)
    adaptor.stack_pointer = Mock(return_value=('sp', 0))
    adaptor.program_counter = Mock(return_value=('pc', 0))



import logging

import voltron
from voltron.api import *

from scruffy.plugin import Plugin

log = logging.getLogger('api')

class APICommandRequest(APIRequest):
    """
    API execute command request.

    {
        "type":         "request",
        "request":      "command"
        "data": {
            "command":  "break list"
        }
    }
    """
    _fields = {'command': True}

    @server_side
    def dispatch(self):
        try:
            output = voltron.debugger.command(self.command)
            res = APICommandResponse()
            res.output = output
        except NoSuchTargetException:
            res = APINoSuchTargetErrorResponse()
        except Exception as e:
            msg = "Exception executing debugger command: {}".format(repr(e))
            log.exception(msg)
            res = APIGenericErrorResponse(msg)

        return res


class APICommandResponse(APISuccessResponse):
    """
    API list targets response.

    {
        "type":         "response",
        "status":       "success",
        "data": {
            "output":   "stuff"
        }
    }
    """
    _fields = {'output': True}

    output = None


class APICommandPlugin(APIPlugin):
    request = "command"
    request_class = APICommandRequest
    response_class = APICommandResponse



from .core import Client


class REPLClient(Client):
    """
    A Voltron client for use in the Python REPL (e.g. Calculon).
    """
    def __getattr__(self, key):
        try:
            res = self.perform_request('registers', registers=[key])
            if res.is_success:
                return res.registers[key]
            else:
                print("Error getting register: {}".format(res.message))
        except Exception as e:
            print("Exception getting register: {}".format(repr(e)))

    def __getitem__(self, key):
        try:
            d = {}
            if isinstance(key, slice):
                d['address'] = key.start
                d['length'] = key.stop - key.start
            else:
                d['address'] = key
                d['length'] = 1

            res = self.perform_request('memory', **d)

            if res.is_success:
                return res.memory
            else:
                print("Error reading memory: {}".format(res.message))
        except Exception as e:
            print("Exception reading memory: {}".format(repr(e)))

    def __setitem__(self, key, value):
        try:
            d = {}
            if isinstance(key, slice):
                d['address'] = key.start
                d['value'] = ((key.stop - key.start) * value)[:key.stop - key.start]
            else:
                d['address'] = key
                d['value'] = value

            res = self.perform_request('write_memory', **d)

            if res.is_success:
                return None
            else:
                print("Error writing memory: {}".format(res.message))
        except Exception as e:
            print("Exception writing memory: {}".format(repr(e)))

    def __call__(self, command):
        try:
            res = self.perform_request('command', command=command)
            if res.is_success:
                return res.output
            else:
                print("Error executing command: {}".format(res.message))
        except Exception as e:
            print("Exception executing command: {}".format(repr(e)))


V = REPLClient()



import six
import struct
import logging
import pygments

from pygments.token import *
from numbers import Number as NumberType
from voltron.core import STRTYPES
from voltron.view import *
from voltron.plugin import *
from voltron.api import *

log = logging.getLogger("view")


class RegisterView (TerminalView):
    FORMAT_INFO = {
        'x86_64': [
            {
                'regs':             ['rax','rbx','rcx','rdx','rbp','rsp','rdi','rsi','rip',
                                     'r8','r9','r10','r11','r12','r13','r14','r15'],
                'label_format':     '{0:3s}',
                'category':         'general',
            },
            {
                'regs':             ['cs','ds','es','fs','gs','ss'],
                'value_format':     SHORT_ADDR_FORMAT_16,
                'category':         'general',
            },
            {
                'regs':             ['rflags'],
                'value_format':     '{}',
                'value_func':       'format_flags',
                'value_colour_en':  False,
                'category':         'general',
            },
            {
                'regs':             ['rflags'],
                'value_format':     '{}',
                'value_func':       'format_jump',
                'value_colour_en':  False,
                'category':         'general',
                'format_name':      'jump'
            },
            {
                'regs':             ['xmm0','xmm1','xmm2','xmm3','xmm4','xmm5','xmm6','xmm7','xmm8',
                                     'xmm9','xmm10','xmm11','xmm12','xmm13','xmm14','xmm15'],
                'value_format':     SHORT_ADDR_FORMAT_128,
                'value_func':       'format_xmm',
                'category':         'sse',
            },
            {
                'regs':             ['st0','st1','st2','st3','st4','st5','st6','st7'],
                'value_format':     '{0:0=20X}',
                'value_func':       'format_fpu',
                'category':         'fpu',
            },
        ],
        'x86': [
            {
                'regs':             ['eax','ebx','ecx','edx','ebp','esp','edi','esi','eip'],
                'label_format':     '{0:3s}',
                'value_format':     SHORT_ADDR_FORMAT_32,
                'category':         'general',
            },
            {
                'regs':             ['cs','ds','es','fs','gs','ss'],
                'value_format':     SHORT_ADDR_FORMAT_16,
                'category':         'general',
            },
            {
                'regs':             ['eflags'],
                'value_format':     '{}',
                'value_func':       'format_flags',
                'value_colour_en':  False,
                'category':         'general',
            },
            {
                'regs':             ['eflags'],
                'value_format':     '{}',
                'value_func':       'format_jump',
                'value_colour_en':  False,
                'category':         'general',
                'format_name':      'jump'
            },
            {
                'regs':             ['xmm0','xmm1','xmm2','xmm3','xmm4','xmm5','xmm6','xmm7'],
                'value_format':     SHORT_ADDR_FORMAT_128,
                'value_func':       'format_xmm',
                'category':         'sse',
            },
            {
                'regs':             ['st0','st1','st2','st3','st4','st5','st6','st7'],
                'value_format':     '{0:0=20X}',
                'value_func':       'format_fpu',
                'category':         'fpu',
            },
        ],
        'arm': [
            {
                'regs':             ['pc','sp','lr','cpsr','r0','r1','r2','r3','r4','r5','r6',
                                    'r7','r8','r9','r10','r11','r12'],
                'label_format':     '{0:>3s}',
                'value_format':     SHORT_ADDR_FORMAT_32,
                'category':         'general',
            }
        ],
        'arm64': [
            {
                'regs':             ['pc', 'sp', 'x0', 'x1', 'x2', 'x3', 'x4', 'x5', 'x6', 'x7', 'x8', 'x9', 'x10',
                                    'x11', 'x12', 'x13', 'x14', 'x15', 'x16', 'x17', 'x18', 'x19', 'x20',
                                    'x21', 'x22', 'x23', 'x24', 'x25', 'x26', 'x27', 'x28', 'x29', 'x30'],
                'label_format':     '{0:3s}',
                'value_format':     SHORT_ADDR_FORMAT_64,
                'category':         'general',
            },
        ],
        'powerpc': [
            {
                'regs':             ['pc','msr','cr','lr', 'ctr',
                                     'r0','r1','r2','r3','r4','r5','r6', 'r7',
                                     'r8','r9','r10','r11','r12','r13','r14', 'r15',
                                     'r16','r17','r18','r19','r20','r21','r22', 'r23',
                                     'r24','r25','r26','r27','r28','r29','r30', 'r31'],
                'label_format':     '{0:>3s}',
                'value_format':     SHORT_ADDR_FORMAT_32,
                'category':         'general',
            }
        ],
    }
    TEMPLATES = {
        'x86_64': {
            'horizontal': {
                'general': (
                    "{raxl} {rax}  {rbxl} {rbx}  {rbpl} {rbp}  {rspl} {rsp}  {rflags}\n"
                    "{rdil} {rdi}  {rsil} {rsi}  {rdxl} {rdx}  {rcxl} {rcx}  {ripl} {rip}\n"
                    "{r8l} {r8}  {r9l} {r9}  {r10l} {r10}  {r11l} {r11}  {r12l} {r12}\n"
                    "{r13l} {r13}  {r14l} {r14}  {r15l} {r15}\n"
                    "{csl} {cs}  {dsl} {ds}  {esl} {es}  {fsl} {fs}  {gsl} {gs}  {ssl} {ss} {jump}"
                ),
                'sse': (
                    "{xmm0l}  {xmm0} {xmm1l}  {xmm1} {xmm2l}  {xmm2}\n"
                    "{xmm3l}  {xmm3} {xmm4l}  {xmm4} {xmm5l}  {xmm5}\n"
                    "{xmm6l}  {xmm6} {xmm7l}  {xmm7} {xmm8l}  {xmm8}\n"
                    "{xmm9l}  {xmm9} {xmm10l} {xmm10} {xmm11l} {xmm11}\n"
                    "{xmm12l} {xmm12} {xmm13l} {xmm13} {xmm14l} {xmm14}\n"
                    "{xmm15l} {xmm15}\n"
                ),
                'fpu': (
                    "{st0l} {st0} {st1l} {st1} {st2l} {st2} {st3l} {st2}\n"
                    "{st4l} {st4} {st5l} {st5} {st6l} {st6} {st7l} {st7}\n"
                )
            },
            'vertical': {
                'general': (
                    "{rflags}\n{jump}\n"
                    "{ripl} {rip}{ripinfo}\n"
                    "{raxl} {rax}{raxinfo}\n"
                    "{rbxl} {rbx}{rbxinfo}\n"
                    "{rbpl} {rbp}{rbpinfo}\n"
                    "{rspl} {rsp}{rspinfo}\n"
                    "{rdil} {rdi}{rdiinfo}\n"
                    "{rsil} {rsi}{rsiinfo}\n"
                    "{rdxl} {rdx}{rdxinfo}\n"
                    "{rcxl} {rcx}{rcxinfo}\n"
                    "{r8l} {r8}{r8info}\n"
                    "{r9l} {r9}{r9info}\n"
                    "{r10l} {r10}{r10info}\n"
                    "{r11l} {r11}{r11info}\n"
                    "{r12l} {r12}{r12info}\n"
                    "{r13l} {r13}{r13info}\n"
                    "{r14l} {r14}{r14info}\n"
                    "{r15l} {r15}{r15info}\n"
                    "{csl}  {cs}  {dsl}  {ds}\n{esl}  {es}  {fsl}  {fs}\n{gsl}  {gs}  {ssl}  {ss}"
                ),
                'sse': (
                    "{xmm0l}  {xmm0}\n{xmm1l}  {xmm1}\n{xmm2l}  {xmm2}\n{xmm3l}  {xmm3}\n"
                    "{xmm4l}  {xmm4}\n{xmm5l}  {xmm5}\n{xmm6l}  {xmm6}\n{xmm7l}  {xmm7}\n"
                    "{xmm8l}  {xmm8}\n{xmm9l}  {xmm9}\n{xmm10l} {xmm10}\n{xmm11l} {xmm11}\n"
                    "{xmm12l} {xmm12}\n{xmm13l} {xmm13}\n{xmm14l} {xmm14}\n{xmm15l} {xmm15}"
                ),
                'fpu': (
                    "{st0l} {st0}\n{st1l} {st1}\n{st2l} {st2}\n{st3l} {st2}\n"
                    "{st4l} {st4}\n{st5l} {st5}\n{st6l} {st6}\n{st7l} {st7}\n"
                )
            }
        },
        'x86': {
            'horizontal': {
                'general': (
                    "{eaxl} {eax}  {ebxl} {ebx}  {ebpl} {ebp}  {espl} {esp}  {eflags}\n"
                    "{edil} {edi}  {esil} {esi}  {edxl} {edx}  {ecxl} {ecx}  {eipl} {eip}\n"
                    "{csl} {cs}  {dsl} {ds}  {esl} {es}  {fsl} {fs}  {gsl} {gs}  {ssl} {ss} {jump}"
                ),
                'sse': (
                    "{xmm0l}  {xmm0} {xmm1l}  {xmm1} {xmm2l}  {xmm2}\n"
                    "{xmm3l}  {xmm3} {xmm4l}  {xmm4} {xmm5l}  {xmm5}\n"
                    "{xmm6l}  {xmm6} {xmm7l}"
                ),
                'fpu': (
                    "{st0l} {st0} {st1l} {st1} {st2l} {st2} {st3l} {st2}\n"
                    "{st4l} {st4} {st5l} {st5} {st6l} {st6} {st7l} {st7}\n"
                )
            },
            'vertical': {
                'general': (
                    "{eflags}\n{jump}\n"
                    "{eipl} {eip}{eipinfo}\n"
                    "{eaxl} {eax}{eaxinfo}\n"
                    "{ebxl} {ebx}{ebxinfo}\n"
                    "{ebpl} {ebp}{ebpinfo}\n"
                    "{espl} {esp}{espinfo}\n"
                    "{edil} {edi}{ediinfo}\n"
                    "{esil} {esi}{esiinfo}\n"
                    "{edxl} {edx}{edxinfo}\n"
                    "{ecxl} {ecx}{ecxinfo}\n"
                    "{csl}  {cs}\n{dsl}  {ds}\n{esl}  {es}\n{fsl}  {fs}\n{gsl}  {gs}\n{ssl}  {ss}"
                ),
                'sse': (
                    "{xmm0l}  {xmm0}\n{xmm1l}  {xmm1}\n{xmm2l}  {xmm2}\n{xmm3l}  {xmm3}\n"
                    "{xmm4l}  {xmm4}\n{xmm5l}  {xmm5}\n{xmm6l}  {xmm6}\n{xmm7l}  {xmm7}\n"
                ),
                'fpu': (
                    "{st0l} {st0}\n{st1l} {st1}\n{st2l} {st2}\n{st3l} {st2}\n"
                    "{st4l} {st4}\n{st5l} {st5}\n{st6l} {st6}\n{st7l} {st7}\n"
                )
            }
        },
        'arm': {
            'horizontal': {
                'general': (
                    "{pcl} {pc} {spl} {sp} {lrl} {lr} {cpsrl} {cpsr}\n"
                    "{r0l} {r0} {r1l} {r1} {r2l} {r2} {r3l} {r3} {r4l} {r4} {r5l} {r5} {r6l} {r6}\n"
                    "{r7l} {r7} {r8l} {r8} {r9l} {r9} {r10l} {r10} {r11l} {r11} {r12l} {r12}"
                ),
            },
            'vertical': {
                'general': (
                    "{pcl} {pc}{pcinfo}\n"
                    "{spl} {sp}{spinfo}\n"
                    "{lrl} {lr}{lrinfo}\n"
                    "{r0l} {r0}{r0info}\n"
                    "{r1l} {r1}{r1info}\n"
                    "{r2l} {r2}{r2info}\n"
                    "{r3l} {r3}{r3info}\n"
                    "{r4l} {r4}{r4info}\n"
                    "{r5l} {r5}{r5info}\n"
                    "{r6l} {r6}{r6info}\n"
                    "{r7l} {r7}{r7info}\n"
                    "{r8l} {r8}{r8info}\n"
                    "{r9l} {r9}{r9info}\n"
                    "{r10l} {r10}{r10info}\n"
                    "{r11l} {r11}{r11info}\n"
                    "{r12l} {r12}{r12info}\n"
                    "{cpsrl}{cpsr}"
                ),
            }
        },
        'powerpc': {
            'horizontal': {
                'general': (
                    "{pcl} {pc} {crl} {cr} {lrl} {lr} {msrl} {msr} {ctrl} {ctr}\n"
                    "{r0l} {r0} {r1l} {r1} {r2l} {r2} {r3l} {r3}\n"
                    "{r4l} {r4} {r5l} {r5} {r6l} {r6} {r7l} {r7}\n"
                    "{r8l} {r8} {r9l} {r9} {r10l} {r10} {r11l} {r11}\n"
                    "{r12l} {r12} {r13l} {r13} {r14l} {r14} {r15l} {r15}\n"
                    "{r16l} {r16} {r17l} {r17} {r18l} {r18} {r19l} {r19}\n"
                    "{r20l} {r20} {r21l} {r21} {r22l} {r22} {r23l} {r23}\n"
                    "{r24l} {r24} {r25l} {r25} {r26l} {r26} {r27l} {r27}\n"
                    "{r28l} {r28} {r29l} {r29} {r30l} {r30} {r31l} {r31}"
                ),
            },
            'vertical': {
                'general': (
                    "{pcl} {pc}{pcinfo}\n"
                    "{crl} {cr}{crinfo}\n"
                    "{lrl} {lr}{lrinfo}\n"
                    "{msrl} {msr}{msrinfo}\n"
                    "{ctrl} {ctr}{ctrinfo}\n"
                    "{r0l} {r0}{r0info}\n"
                    "{r1l} {r1}{r1info}\n"
                    "{r2l} {r2}{r2info}\n"
                    "{r3l} {r3}{r3info}\n"
                    "{r4l} {r4}{r4info}\n"
                    "{r5l} {r5}{r5info}\n"
                    "{r6l} {r6}{r6info}\n"
                    "{r7l} {r7}{r7info}\n"
                    "{r8l} {r8}{r8info}\n"
                    "{r9l} {r9}{r9info}\n"
                    "{r10l} {r10}{r10info}\n"
                    "{r11l} {r11}{r11info}\n"
                    "{r12l} {r12}{r12info}\n"
                    "{r13l} {r13}{r13info}\n"
                    "{r14l} {r14}{r14info}\n"
                    "{r15l} {r15}{r15info}\n"
                    "{r16l} {r16}{r16info}\n"
                    "{r17l} {r17}{r17info}\n"
                    "{r18l} {r18}{r18info}\n"
                    "{r19l} {r19}{r19info}\n"
                    "{r20l} {r20}{r20info}\n"
                    "{r21l} {r21}{r21info}\n"
                    "{r22l} {r22}{r22info}\n"
                    "{r23l} {r23}{r23info}\n"
                    "{r24l} {r24}{r24info}\n"
                    "{r25l} {r25}{r25info}\n"
                    "{r26l} {r26}{r26info}\n"
                    "{r27l} {r27}{r27info}\n"
                    "{r28l} {r28}{r28info}\n"
                    "{r29l} {r29}{r29info}\n"
                    "{r30l} {r30}{r30info}\n"
                    "{r31l} {r31}{r31info}"
                ),
            }
        },
        'arm64': {
            'horizontal': {
                'general': (
                    "{pcl} {pc}{pcinfo}\n"
                    "{spl} {sp}{spinfo}\n"
                    "{x0l} {x0}{x0info}\n"
                    "{x1l} {x1}{x1info}\n"
                    "{x2l} {x2}{x2info}\n"
                    "{x3l} {x3}{x3info}\n"
                    "{x4l} {x4}{x4info}\n"
                    "{x5l} {x5}{x5info}\n"
                    "{x6l} {x6}{x6info}\n"
                    "{x7l} {x7}{x7info}\n"
                    "{x8l} {x8}{x8info}\n"
                    "{x9l} {x9}{x9info}\n"
                    "{x10l} {x10}{x10info}\n"
                    "{x11l} {x11}{x11info}\n"
                    "{x12l} {x12}{x12info}\n"
                    "{x13l} {x13}{x13info}\n"
                    "{x14l} {x14}{x14info}\n"
                    "{x15l} {x15}{x15info}\n"
                    "{x16l} {x16}{x16info}\n"
                    "{x17l} {x17}{x17info}\n"
                    "{x18l} {x18}{x18info}\n"
                    "{x19l} {x19}{x19info}\n"
                    "{x20l} {x20}{x20info}\n"
                    "{x21l} {x21}{x21info}\n"
                    "{x22l} {x22}{x22info}\n"
                    "{x23l} {x23}{x23info}\n"
                    "{x24l} {x24}{x24info}\n"
                    "{x25l} {x25}{x25info}\n"
                    "{x26l} {x26}{x26info}\n"
                    "{x27l} {x27}{x27info}\n"
                    "{x28l} {x28}{x28info}\n"
                    "{x29l} {x29}{x29info}\n"
                    "{x30l} {x30}{x30info}\n"
                ),
            },
            'vertical': {
                'general': (
                    "{pcl} {pc}{pcinfo}\n"
                    "{spl} {sp}{spinfo}\n"
                    "{x0l} {x0}{x0info}\n"
                    "{x1l} {x1}{x1info}\n"
                    "{x2l} {x2}{x2info}\n"
                    "{x3l} {x3}{x3info}\n"
                    "{x4l} {x4}{x4info}\n"
                    "{x5l} {x5}{x5info}\n"
                    "{x6l} {x6}{x6info}\n"
                    "{x7l} {x7}{x7info}\n"
                    "{x8l} {x8}{x8info}\n"
                    "{x9l} {x9}{x9info}\n"
                    "{x10l} {x10}{x10info}\n"
                    "{x11l} {x11}{x11info}\n"
                    "{x12l} {x12}{x12info}\n"
                    "{x13l} {x13}{x13info}\n"
                    "{x14l} {x14}{x14info}\n"
                    "{x15l} {x15}{x15info}\n"
                    "{x16l} {x16}{x16info}\n"
                    "{x17l} {x17}{x17info}\n"
                    "{x18l} {x18}{x18info}\n"
                    "{x19l} {x19}{x19info}\n"
                    "{x20l} {x20}{x20info}\n"
                    "{x21l} {x21}{x21info}\n"
                    "{x22l} {x22}{x22info}\n"
                    "{x23l} {x23}{x23info}\n"
                    "{x24l} {x24}{x24info}\n"
                    "{x25l} {x25}{x25info}\n"
                    "{x26l} {x26}{x26info}\n"
                    "{x27l} {x27}{x27info}\n"
                    "{x28l} {x28}{x28info}\n"
                    "{x29l} {x29}{x29info}\n"
                    "{x30l} {x30}{x30info}\n"
                ),
            }
        }
    }
    FLAG_BITS = {'c': 0, 'p': 2, 'a': 4, 'z': 6, 's': 7, 't': 8, 'i': 9, 'd': 10, 'o': 11}
    FLAG_TEMPLATE = "{o} {d} {i} {t} {s} {z} {a} {p} {c}"
    XMM_INDENT = 7
    last_regs = None
    last_flags = None

    @classmethod
    def configure_subparser(cls, subparsers):
        sp = subparsers.add_parser('registers', help='register values', aliases=('r', 'reg', 'register'))
        VoltronView.add_generic_arguments(sp)
        sp.set_defaults(func=RegisterView)
        g = sp.add_mutually_exclusive_group()
        g.add_argument('--horizontal', '-o', dest="orientation", action='store_const', const="horizontal",
                       help='horizontal orientation')
        g.add_argument('--vertical', '-v', dest="orientation", action='store_const', const="vertical",
                       help='vertical orientation (default)')
        sp.add_argument('--general', '-g', dest="sections", action='append_const', const="general",
                        help='show general registers')
        sp.add_argument('--no-general', '-G', dest="sections", action='append_const', const="no_general",
                        help='hide general registers')
        sp.add_argument('--sse', '-s', dest="sections", action='append_const', const="sse", help='show sse registers')
        sp.add_argument('--no-sse', '-S', dest="sections", action='append_const', const="no_sse",
                        help='hide sse registers')
        sp.add_argument('--fpu', '-p', dest="sections", action='append_const', const="fpu", help='show fpu registers')
        sp.add_argument('--no-fpu', '-P', dest="sections", action='append_const', const="no_fpu",
                        help='hide fpu registers')
        sp.add_argument('--hide-info', '-I',dest="hide_info", action='store_true', help='hide extra info (pointer derefs, ascii) for registers',
                        default=False)

    def __init__(self, *args, **kwargs):
        super(RegisterView, self).__init__(*args, **kwargs)
        self.str_upper = str.upper

    def apply_cli_config(self):
        super(RegisterView, self).apply_cli_config()
        if self.args.orientation is not None:
            self.config.orientation = self.args.orientation
        if self.args.sections is not None:
            a = filter(lambda x: 'no_' + x not in self.args.sections and not x.startswith('no_'), list(self.config.sections) + self.args.sections)
            config_sections = []
            for sec in a:
                if sec not in config_sections:
                    config_sections.append(sec)
            self.config.sections = config_sections

    def build_requests(self):
        return [
            api_request('targets', block=self.block),
            api_request('disassemble', count=1, block=self.block),
            api_request('registers', block=self.block)
        ]

    def render(self, results):
        error = None
        t_res, d_res, r_res = results
        formatter = pygments.formatters.get_formatter_by_name(self.config.format.pygments_formatter,
                                                              style=self.config.format.pygments_style)

        def format(tok, tik=None):
            if tik:
                tok = (tok, tik)
            if isinstance(tok, tuple):
                return pygments.format([tok], formatter)
            else:
                return pygments.format(tok, formatter)
        self.f = format

        if t_res and t_res.is_error:
            error = t_res.message
        elif t_res is None or t_res and len(t_res.targets) == 0:
            error = "No such target"
        else:
            arch = t_res.targets[0]['arch']
            self.curr_arch = arch

            # ensure the architecture is supported
            if arch not in self.FORMAT_INFO:
                error = "Architecture '{}' not supported".format(arch)
            else:
                # get next instruction
                try:
                    self.curr_inst = d_res.disassembly.strip().split('\n')[-1].split(':')[1].strip()
                except:
                    self.curr_inst = None

                # get registers for target
                if r_res.is_error:
                    error = r_res.message

        # if everything is ok, render the view
        if not error:
            # Build template
            template = '\n'.join(map(lambda x: self.TEMPLATES[arch][self.config.orientation][x], self.config.sections))

            # Process formatting settings
            addr_size = t_res.targets[0]['addr_size']
            data = defaultdict(lambda: 'n/a')
            data.update(r_res.registers)
            formats = self.FORMAT_INFO[arch]
            formatted = {}
            for fmt in formats:
                # Apply defaults where they're missing
                fmt = dict(list(self.config.format.items()) + list(fmt.items()))

                # Format the data for each register
                for reg in fmt['regs']:
                    # Format the label
                    label = fmt['label_format'].format(reg)
                    if fmt['label_func'] != None:
                        formatted[reg + 'l'] = getattr(self, fmt['label_func'])(str(label))
                    if fmt['label_colour_en']:
                        formatted[reg + 'l'] = format(Name.Label, formatted[reg + 'l'])

                    # Format the value
                    val = data[reg]
                    if isinstance(val, STRTYPES):
                        temp = fmt['value_format'].format(0)
                        if len(val) < len(temp):
                            val += (len(temp) - len(val)) * ' '
                        formatted_reg = format(Text, val)
                    else:
                        token = Text
                        if self.last_regs is None or self.last_regs is not None and val != self.last_regs[reg]:
                            token = Error
                        formatted_reg = val
                        if fmt['value_format'] != None and isinstance(formatted_reg, NumberType):
                            formatted_reg = fmt['value_format'].format(formatted_reg)
                        if fmt['value_func'] != None:
                            if isinstance(fmt['value_func'], STRTYPES):
                                formatted_reg = getattr(self, fmt['value_func'])(formatted_reg)
                            else:
                                formatted_reg = fmt['value_func'](formatted_reg)
                        formatted_reg = format(token, formatted_reg)
                    if fmt['format_name'] == None:
                        formatted[reg] = formatted_reg
                    else:
                        formatted[fmt['format_name']] = formatted_reg

                    # Format the info
                    if not self.args.hide_info:
                        info = ""
                        try:
                            l = {2: 'H', 4: 'L', 8: 'Q'}[t_res.targets[0]['addr_size']]
                            x = '{}{}'.format(('<' if t_res.targets[0]['byte_order'] == 'little' else '>'), l)
                            chunk = struct.pack(x, data[reg])
                            printable_filter = ''.join([(len(repr(chr(x))) == 3) and chr(x) or '.' for x in range(256)])
                            ascii_str = ''.join(["%s" % ((x <= 127 and printable_filter[x]) or '.') for x in six.iterbytes(chunk)])
                            pipe = format(Punctuation, '|')
                            info += ' ' + pipe + ' ' + ascii_str + ' ' + pipe
                        except:
                            pass
                        try:
                            fmtd = [(Punctuation, ' => ')]
                            for t, item in r_res.deref[reg][1:]:
                                if t == "pointer":
                                    fmtd.append((Number.Hex, self.format_address(item, size=addr_size, pad=False)))
                                elif t == "string":
                                    item = item.replace('\n', '\\n')
                                    fmtd.append((String.Double, '"' + item + '"'))
                                elif t == "unicode":
                                    item = item.replace('\n', '\\n')
                                    fmtd.append((String.Double, 'u"' + item + '"'))
                                elif t == "symbol":
                                    fmtd.append((Name.Function, '`' + item + '`'))
                                elif t == "circular":
                                    fmtd.append((Text, '(circular)'))
                                fmtd.append((Punctuation, ' => '))
                            if len(r_res.deref[reg][1:]):
                                info += format(fmtd[:-1])
                        except KeyError:
                            pass
                        except IndexError:
                            pass
                    else:
                        info = ''
                    formatted[reg + 'info'] = info

            # Prepare output
            log.debug('Formatted: ' + str(formatted))
            self.body = template.format(**formatted)

            # Store the regs
            self.last_regs = data
        else:
            # Set body to error message if appropriate
            self.body = format(Error, error)

        # Prepare headers and footers
        height, width = self.window_size()
        self.title = '[regs:{}]'.format('|'.join(self.config.sections))
        if len(self.title) > width:
            self.title = '[regs]'

        # Call parent's render method
        super(RegisterView, self).render(results)

    def format_address(self, address, size=8, pad=True, prefix='0x'):
        fmt = '{:' + ('0=' + str(size * 2) if pad else '') + 'X}'
        addr_str = fmt.format(address)
        if prefix:
            addr_str = prefix + addr_str
        return addr_str

    def format_flags(self, val):
        values = {}

        # Get formatting info for flags
        if self.curr_arch == 'x86_64':
            reg = 'rflags'
        elif self.curr_arch == 'x86':
            reg = 'eflags'
        fmt = dict(list(self.config.format.items()) + list(list(filter(lambda x: reg in x['regs'], self.FORMAT_INFO[self.curr_arch]))[0].items()))

        # Handle each flag bit
        val = int(val, 10)
        formatted = {}
        for flag in self.FLAG_BITS.keys():
            values[flag] = (val & (1 << self.FLAG_BITS[flag]) > 0)
            log.debug("Flag {} value {} (for flags 0x{})".format(flag, values[flag], val))
            formatted[flag] = str.upper(flag) if values[flag] else flag
            if self.last_flags is not None and self.last_flags[flag] != values[flag]:
                token = Error
            else:
                token = Text
            formatted[flag] = self.f(token, formatted[flag])

        # Store the flag values for comparison
        self.last_flags = values

        # Format with template
        flags = self.FLAG_TEMPLATE.format(**formatted)

        return self.f(Text, '[ ') + flags + self.f(Text, ' ]')

    def format_jump(self, val):
        # Grab flag bits
        val = int(val, 10)
        values = {}
        for flag in self.FLAG_BITS.keys():
            values[flag] = (val & (1 << self.FLAG_BITS[flag]) > 0)

        # If this is a jump instruction, see if it will be taken
        j = None
        if self.curr_inst:
            inst = self.curr_inst.split()[0]
            if inst in ['ja', 'jnbe']:
                if not values['c'] and not values['z']:
                    j = (True, '!c && !z')
                else:
                    j = (False, 'c || z')
            elif inst in ['jae', 'jnb', 'jnc']:
                if not values['c']:
                    j = (True, '!c')
                else:
                    j = (False, 'c')
            elif inst in ['jb', 'jc', 'jnae']:
                if values['c']:
                    j = (True, 'c')
                else:
                    j = (False, '!c')
            elif inst in ['jbe', 'jna']:
                if values['c'] or values['z']:
                    j = (True, 'c || z')
                else:
                    j = (False, '!c && !z')
            elif inst in ['jcxz', 'jecxz', 'jrcxz']:
                if self.get_arch() == 'x64':
                    cx = regs['rcx']
                elif self.get_arch() == 'x86':
                    cx = regs['ecx']
                if cx == 0:
                    j = (True, cx + '==0')
                else:
                    j = (False, cx + '!=0')
            elif inst in ['je', 'jz']:
                if values['z']:
                    j = (True, 'z')
                else:
                    j = (False, '!z')
            elif inst in ['jnle', 'jg']:
                if not values['z'] and values['s'] == values['o']:
                    j = (True, '!z && s==o')
                else:
                    j = (False, 'z || s!=o')
            elif inst in ['jge', 'jnl']:
                if values['s'] == values['o']:
                    j = (True, 's==o')
                else:
                    j = (False, 's!=o')
            elif inst in ['jl', 'jnge']:
                if values['s'] == values['o']:
                    j = (False, 's==o')
                else:
                    j = (True, 's!=o')
            elif inst in ['jle', 'jng']:
                if values['z'] or values['s'] == values['o']:
                    j = (True, 'z || s==o')
                else:
                    j = (False, '!z && s!=o')
            elif inst in ['jne', 'jnz']:
                if not values['z']:
                    j = (True, '!z')
                else:
                    j = (False, 'z')
            elif inst in ['jno']:
                if not values['o']:
                    j = (True, '!o')
                else:
                    j = (False, 'o')
            elif inst in ['jnp', 'jpo']:
                if not values['p']:
                    j = (True, '!p')
                else:
                    j = (False, 'p')
            elif inst in ['jns']:
                if not values['s']:
                    j = (True, '!s')
                else:
                    j = (False, 's')
            elif inst in ['jo']:
                if values['o']:
                    j = (True, 'o')
                else:
                    j = (False, '!o')
            elif inst in ['jp', 'jpe']:
                if values['p']:
                    j = (True, 'p')
                else:
                    j = (False, '!p')
            elif inst in ['js']:
                if values['s']:
                    j = (True, 's')
                else:
                    j = (False, '!s')

        # Construct message
        if j is not None:
            taken, reason = j
            if taken:
                jump = 'Jump ({})'.format(reason)
            else:
                jump = '!Jump ({})'.format(reason)
        else:
            jump = ''

        # Pad out
        jump = '{:^19}'.format(jump)

        # Colour
        if j is not None:
            jump = self.f(Error, jump)
        else:
            jump = self.f(Text, jump)

        return self.f(Text, '[') + jump + self.f(Text, ']')

    def format_xmm(self, val):
        if self.config.orientation == 'vertical':
            height, width = self.window_size()
            if width < len(SHORT_ADDR_FORMAT_128.format(0)) + self.XMM_INDENT:
                return val[:16] + '\n' + ' ' * self.XMM_INDENT + val[16:]
            else:
                return val[:16] + ':' + val[16:]
        else:
            return val

    def format_fpu(self, val):
        if self.config.orientation == 'vertical':
            return val
        else:
            return val


class RegisterViewPlugin(ViewPlugin):
    plugin_type = 'view'
    name = 'register'
    view_class = RegisterView



import voltron
import logging

from voltron.api import *

log = logging.getLogger('api')


class APIRegistersRequest(APIRequest):
    """
    API state request.

    {
        "type":         "request",
        "request":      "registers",
        "data": {
            "target_id": 0,
            "thread_id": 123456,
            "registers": ['rsp']
        }
    }

    `target_id` and `thread_id` are optional. If not present, the currently
    selected target and thread will be used.

    `registers` is optional. If it is not included all registers will be
    returned.
    """
    _fields = {'target_id': False, 'thread_id': False, 'registers': False}

    target_id = 0
    thread_id = None
    registers = []

    @server_side
    def dispatch(self):
        try:
            regs = voltron.debugger.registers(target_id=self.target_id, thread_id=self.thread_id, registers=self.registers)
            res = APIRegistersResponse()
            res.registers = regs
            res.deref = {}
            for reg, val in regs.items():
                try:
                    if val > 0:
                        try:
                            res.deref[reg] = voltron.debugger.dereference(pointer=val)
                        except:
                            res.deref[reg] = []
                    else:
                        res.deref[reg] = []
                except TypeError:
                    res.deref[reg] = []
        except TargetBusyException:
            res = APITargetBusyErrorResponse()
        except NoSuchTargetException:
            res = APINoSuchTargetErrorResponse()
        except Exception as e:
            msg = "Exception getting registers from debugger: {}".format(repr(e))
            log.exception(msg)
            res = APIGenericErrorResponse(msg)

        return res


class APIRegistersResponse(APISuccessResponse):
    """
    API status response.

    {
        "type":         "response",
        "status":       "success",
        "data": {
            "registers": { "rip": 0x12341234, ... },
            "deref": {"rip": [(pointer, 0x12341234), ...]}
        }
    }
    """
    _fields = {'registers': True, 'deref': False}


class APIRegistersPlugin(APIPlugin):
    request = 'registers'
    request_class = APIRegistersRequest
    response_class = APIRegistersResponse



from __future__ import print_function

import struct
import string
import logging
import threading

from voltron.api import *
from voltron.plugin import *
from voltron.dbg import *

try:
    import vtrace
    import vdb
    import envi
    HAVE_VDB = True
except ImportError:
    HAVE_VDB = False

log = logging.getLogger('debugger')

if HAVE_VDB:

    class NotAStringError(Exception):
        pass

    class FailedToReadMemoryError(Exception):
        pass

    class VDBAdaptor(DebuggerAdaptor):
        """
        The interface with an instance of VDB
        """

        archs = {
            "i386": "x86",
            "amd64": "x86_64",
            "arm": "arm",
        }

        sizes = {
            'x86': 4,
            'x86_64': 8,
            'arm': 4
        }

        reg_names = {
            "x86_64": {
                "pc": "rip",
                "sp": "rsp",
            },
            "x86": {
                "pc": "eip",
                "sp": "esp",
            }
        }

        def __init__(self, host, *args, **kwargs):
            self.listeners = []
            self.host_lock = threading.RLock()
            self._vdb = host
            self._vtrace = vtrace

        def version(self):
            """
            Get the debugger's version.

            Returns a string containing the debugger's version
            (e.g. 'GNU gdb (GDB) 7.8')
            """
            return "VDB/version-unknown"

        def _target(self, target_id=0):
            """
            Return information about the specified target.

            Returns data in the following structure:
            {
                "id":       0,         # ID that can be used in other funcs
                "file":     "/bin/ls", # target's binary file
                "arch":     "x86_64",  # target's architecture
                "state:     "stopped"  # state
            }
            """
            d = {}
            d["id"] = 0
            d["state"] = self._state()
            d["file"] = self._vdb.getTrace().metadata['ExeName']
            d["arch"] = self.get_arch()
            d['byte_order'] = self.get_byte_order()
            d['addr_size'] = self.get_addr_size()
            return d

        @lock_host
        def target(self, target_id=0):
            """
            Return information about the current inferior.

            `target_id` is ignored.
            """
            return self._target()

        @lock_host
        def targets(self, target_ids=None):
            """
            Return information about the debugger's current targets.

            `target_ids` is ignored. Only the current target is returned. This
            method is only implemented to maintain API compatibility with the
            LLDBAdaptor.
            """
            return [self._target()]

        @validate_target
        @lock_host
        def state(self, target_id=0):
            """
            Get the state of a given target.
            `target_id` is ignored.
            """
            return self._state()

        @validate_busy
        @validate_target
        @lock_host
        def registers(self, target_id=0, thread_id=None, registers=[]):
            """
            Get the register values for a given target/thread.
            `target_id` is ignored.
            """
            arch = self.get_arch()

            if arch in self.reg_names:
                if 'pc' in registers:
                    registers.remove('pc')
                    registers.append(self.reg_names[arch]['pc'])
                if 'sp' in registers:
                    registers.remove('sp')
                    registers.append(self.reg_names[arch]['sp'])
            else:
                raise Exception("Unsupported architecture: {}".format(target['arch']))

            if registers != []:
                regs = {}
                for reg in registers:
                    regs[reg] = self.get_register(reg)
            else:
                log.debug('Getting registers for arch {}'.format(arch))
                if arch == "x86_64":
                    regs = self.get_registers_x86_64()
                elif arch == "x86":
                    regs = self.get_registers_x86()
                elif arch == "arm":
                    regs = self.get_registers_arm()
                else:
                    raise UnknownArchitectureException()

            return regs

        @validate_busy
        @validate_target
        @lock_host
        def stack_pointer(self, target_id=0, thread_id=None):
            """
            Get the value of the stack pointer register.
            `target_id` is ignored.
            """
            arch = self.get_arch()
            if arch in self.reg_names:
                sp_name = self.reg_names[arch]['sp']
                sp = self.get_register(sp_name)
            else:
                raise UnknownArchitectureException()

            return sp_name, sp

        @validate_busy
        @validate_target
        @lock_host
        def program_counter(self, target_id=0, thread_id=None):
            """
            Get the value of the program counter register.
            `target_id` is ignored.
            """
            arch = self.get_arch()
            if arch in self.reg_names:
                pc_name = self.reg_names[arch]['pc']
                pc = self.get_register(pc_name)
            else:
                raise UnknownArchitectureException()

            return pc_name, pc

        @validate_busy
        @validate_target
        @lock_host
        def memory(self, address, length, target_id=0):
            """
            Get the register values for .
            Raises `FailedToReadMemoryError` if... that happens.

            `address` is the address at which to start reading
            `length` is the number of bytes to read
            `target_id` is ignored.
            """
            log.debug('Reading 0x{:x} bytes of memory at 0x{:x}'.format(length, address))
            t = self._vdb.getTrace()
            try:
                return t.readMemory(address, length)
            except:
                raise FailedToReadMemoryError()

        @validate_busy
        @validate_target
        @lock_host
        def stack(self, length, target_id=0, thread_id=None):
            """
            Get the register values for .

            `length` is the number of bytes to read
            `target_id` is a target ID (or None for the first target)
            `thread_id` is a thread ID (or None for the selected thread)
            """
            # get the stack pointer
            sp_name, sp = self.stack_pointer(target_id=target_id, thread_id=thread_id)

            # read memory
            memory = self.memory(sp, length, target_id=target_id)

            return memory

        def _get_n_opcodes_length(self, address, count):
            """
            Get the number of bytes used to represent the `n` instructions
              at `address`.

            `address` is the starting address of the sequence of instructions.
            `count` is the number of instructions to decode.
            """
            length = 0
            t = self._vdb.getTrace()
            arch = self._vdb.arch.getArchId()
            for i in xrange(count):
                op = t.parseOpcode(address + length, arch=arch)
                length += op.size
            return length

        @validate_busy
        @validate_target
        @lock_host
        def disassemble(self, target_id=0, address=None, count=16):
            """
            Get a disassembly of the instructions at the given address.

            `address` is the address at which to disassemble. If None, the
            current program counter is used.
            `count` is the number of instructions to disassemble.
            """
            if address == None:
                pc_name, address = self.program_counter(target_id=target_id)

            length = self._get_n_opcodes_length(address, count)
            can = envi.memcanvas.StringMemoryCanvas(self._vdb.memobj, self._vdb.symobj)
            can.renderMemory(address, length, self._vdb.opcoderend)
            return str(can)

        def _get_ascii_string(self, address, min_length=4, max_length=32):
            """
            Get the ASCII string of length at least `min_length`, but
             not more than `max_length` of it, or raise
             `NotAStringError` if it doesnt look like an ASCII string.
            """
            cs = []
            for i in xrange(max_length):
                try:
                    c = self.memory(address + i, 1)[0]
                except FailedToReadMemoryError:
                    break
                if ord(c) == 0:
                    break
                elif c not in string.printable:
                    break
                else:
                    cs.append(c)

            if len(cs) >= min_length:
                return "".join(cs)
            else:
                raise NotAStringError()

        def _get_unicode_string(self, address, min_length=4, max_length=32):
            """
            Get the *simple* Unicode string of length at least `min_length`
             characters, but not more than `max_length` characters of it,
             or raise `NotAStringError` if it doesnt look like a
             *simple* Unicode string.

            *simple* Unicode is ASCII with interspersed NULLs
            """
            cs = []
            for i in xrange(max_length):
                try:
                    b = self.memory(address + (i * 2), 2)
                except FailedToReadMemoryError:
                    break

                # need every other byte to be a NULL
                if ord(b[1]) != 0:
                    break

                c = b[0]
                if ord(c) == 0:
                    break
                elif c not in string.printable:
                    break
                else:
                    cs.append(c)

            if len(cs) >= min_length:
                return "".join(cs)
            else:
                raise NotAStringError()

        @validate_busy
        @validate_target
        @lock_host
        def dereference(self, pointer, target_id=0):
            """
            Recursively dereference a pointer for display
            `target_id` is ignored.
            """
            fmt = ('<' if self.get_byte_order() == 'little' else '>') + {2: 'H', 4: 'L', 8: 'Q'}[self.get_addr_size()]

            addr = pointer
            chain = []

            # recursively dereference
            while True:
                try:
                    mem = self.memory(addr, self.get_addr_size())
                except FailedToReadMemoryError:
                    break
                except Exception as e:
                    print(e)
                    print(type(e))
                    print(e.__class__.__name__)
                    break
                log.debug("read mem: {}".format(mem))
                (ptr,) = struct.unpack(fmt, mem)
                if ptr in chain:
                    break
                chain.append(('pointer', addr))
                addr = ptr

            # get some info for the last pointer
            # first try to resolve a symbol context for the address
            p, addr = chain[-1]
            output = self._vdb.reprPointer(addr)
            if "Who knows?!?!!?" not in output:
                chain.append(('symbol', output))
                log.debug("symbol context: {}".format(str(chain[-1])))
            else:
                log.debug("no symbol context")
                try:
                    chain.append(("string", self._get_ascii_string(addr)))
                except NotAStringError:
                    try:
                        chain.append(("string", self._get_unicode_string(addr)))
                    except NotAStringError:
                        pass

            log.debug("chain: {}".format(chain))
            return chain

        @lock_host
        def command(self, command=None):
            """
            Execute a command in the debugger.

            `command` is the command string to execute.
            """
            if command:
                # well, this is hacky...
                # hook the canvas to capture a command's output
                oldcan = self._vdb.canvas
                newcan = envi.memcanvas.StringMemoryCanvas(self._vdb.memobj, self._vdb.symobj)
                try:
                    self._vdb.canvas = newcan
                    self._vdb.onecmd(command)
                finally:
                    self._vdb.canvas = oldcan
                return str(newcan).rstrip("\n")
            else:
                raise Exception("No command specified")

            return res

        @lock_host
        def disassembly_flavor(self):
            """
            Return the disassembly flavor setting for the debugger.

            Returns 'intel' or 'att'
            """
            return "intel"

        def capabilities(self):
            """
            Return a list of the debugger's capabilities.

            Thus far only the 'async' capability is supported. This indicates
            that the debugger host can be queried from a background thread,
            and that views can use non-blocking API requests without queueing
            requests to be dispatched next time the debugger stops.
            """
            return ['async']

        #
        # Private functions
        #

        def _state(self):
            """
            Get the state of a given target. Internal use.
            """
            if not self._vdb.getTrace().isAttached():
                state = "invalid"
            else:
                if self._vdb.getTrace().isRunning():
                    state = "running"
                else:
                    state = "stopped"
            return state

        def get_registers(self):
            return self._vdb.getTrace().getRegisters()

        def get_register(self, reg_name):
            return self.get_registers()[reg_name]

        def get_registers_x86_64(self):
            return self.get_registers()

        def get_registers_x86(self):
            return self.get_registers()

        def get_registers_arm(self):
            return self.get_registers()

        def get_registers_sse(self, num=8):
            sse = {}
            for k, v in self.get_registers().items():
                if k.startswith("xmm"):
                    sse[k] = v
            return sse

        def get_registers_fpu(self):
            fpu = {}
            for k, v in self.get_registers().items():
                if k.startswith("st"):
                    fpu[k] = v
            return fpu

        def get_next_instruction(self):
            dis = self.disassemble(address=self.program_counter()[1], count=1)
            return dis.partition("\n")[0].strip()

        def get_arch(self):
            arch = self._vdb.getTrace().getMeta("Architecture")
            return self.archs[arch]

        def get_addr_size(self):
            arch = self.get_arch()
            return self.sizes[arch]

        def get_byte_order(self):
            return "little"


    class VDBCommand(DebuggerCommand, vtrace.Notifier):
        """
        Debugger command class for VDB
        """
        def __init__(self, host):
            """
            vdb is the debugger instance
            vtrace is the vtrace module?
            """
            super(VDBCommand, self).__init__()
            self._vdb = host
            self._vtrace = vtrace
            self.register_hooks()

        def invoke(self, arg, from_tty):
            self.handle_command(arg)

        def register_hooks(self):
            self._vdb.registerNotifier(vtrace.NOTIFY_ALL, self)

        def unregister_hooks(self):
            self._vdb.deregisterNotifier(vtrace.NOTIFY_ALL, self)

        def notify(self, event, trace):
            if event == self._vtrace.NOTIFY_DETACH:
                self.exit_handler(event)
            elif event == self._vtrace.NOTIFY_EXIT:
                self.exit_handler(event)
            elif event == self._vtrace.NOTIFY_BREAK:
                self.stop_handler(event)
            elif event == self._vtrace.NOTIFY_STEP:
                self.stop_handler(event)
            elif event == self._vtrace.NOTIFY_CONTINUE:
                self.cont_handler(event)

        def stop_handler(self, event):
            self.adaptor.update_state()
            voltron.server.dispatch_queue()
            log.debug('Inferior stopped')

        def exit_handler(self, event):
            log.debug('Inferior exited')
            voltron.server.cancel_queue()
            voltron.server.stop()

        def cont_handler(self, event):
            log.debug('Inferior continued')


    class VDBAdaptorPlugin(DebuggerAdaptorPlugin):
        host = 'vdb'
        adaptor_class = VDBAdaptor
        command_class = VDBCommand



import voltron
from voltron.api import *


import logging

import voltron
from voltron.api import *

from scruffy.plugin import Plugin

log = logging.getLogger('api')


class APIBreakpointsRequest(APIRequest):
    """
    API breakpoints request.

    {
        "type":         "request",
        "request":      "breakpoints"
    }
    """
    @server_side
    def dispatch(self):
        try:
            bps = voltron.debugger.breakpoints()
            res = APIBreakpointsResponse(breakpoints=bps)
        except NoSuchTargetException:
            res = APINoSuchTargetErrorResponse()
        except Exception as e:
            msg = "Exception getting breakpoints: {}".format(repr(e))
            log.exception(msg)
            res = APIGenericErrorResponse(msg)

        return res


class APIBreakpointsResponse(APISuccessResponse):
    """
    API breakpoints response.

    {
        "type":         "response",
        "status":       "success",
        "data": {
            "breakpoints": [
                {
                    "id":           1,
                    "enabled":      True,
                    "one_shot":     False,
                    "hit_count":    5,
                    "locations": [
                        {
                            "address":  0x100000cf0,
                            "name":     'main'
                        }
                    ]
                }
            ]
        }
    }
    """
    _fields = {'breakpoints': True}

    breakpoints = []


class APIBreakpointsPlugin(APIPlugin):
    request = "breakpoints"
    request_class = APIBreakpointsRequest
    response_class = APIBreakpointsResponse



from __future__ import print_function

import sys
import platform
import textwrap
from subprocess import check_output
from setuptools import setup, find_packages


def check_install():
    """
    Try to detect the two most common installation errors:

    1. Installing on macOS using a Homebrew version of Python
    2. Installing on Linux using Python 2 when GDB is linked with Python 3
    """
    if platform.system() == 'Darwin' and sys.executable != '/usr/bin/python':
        # send warning text to stderr so we can capture version string in a shell script like
        # version=$(python3 setup.py --version)
        print("*" * 79, file=sys.stderr)
        print(textwrap.fill(
            "WARNING: You are not using the version of Python included with "
            "macOS. If you intend to use Voltron with the LLDB included "
            "with Xcode, or GDB installed with Homebrew, it will not work "
            "unless it is installed using the system's default Python. If "
            "you intend to use Voltron with a debugger installed by some "
            "other method, it may be safe to ignore this warning. See the "
            "following documentation for more detailed installation "
            "instructions: "
            "https://github.com/snare/voltron/wiki/Installation", 79), file=sys.stderr)
        print("*" * 79, file=sys.stderr)
    elif platform.system() == 'Linux':
        try:
            output = check_output([
                "gdb", "-batch", "-q", "--nx", "-ex",
                "pi print(sys.version_info.major)"
            ]).decode("utf-8")
            gdb_python = int(output)

            if gdb_python != sys.version_info.major:
                print("*" * 79, file=sys.stderr)
                print(textwrap.fill(
                    "WARNING: You are installing Voltron using Python {0}.x "
                    "and GDB is linked with Python {1}.x. GDB will not be "
                    "able to load Voltron. Please install using Python {1} "
                    "if you intend to use Voltron with the copy of GDB that "
                    "is installed. See the following documentation for more "
                    "detailed installation instructions: "
                    "https://github.com/snare/voltron/wiki/Installation"
                    .format(sys.version_info.major, gdb_python), 79), file=sys.stderr)
                print("*" * 79, file=sys.stderr)
        except:
            pass


check_install()


requirements = [
    'scruffington>=0.3.9',
    'flask',
    'flask_restful',
    'blessed',
    'pygments',
    'requests',
    'requests_unixsocket',
    'six',
    'pysigset',
    'pygments',
]
if sys.platform == 'win32':
    requirements.append('cursor')


setup(
    name="voltron",
    version="0.1.8",
    author="snare",
    author_email="snare@ho.ax",
    description="A debugger UI",
    license="MIT",
    keywords="voltron debugger ui gdb lldb vdb "
             "vivisect vtrace windbg cdb pykd",
    url="https://github.com/snare/voltron",
    packages=find_packages(exclude=['tests', 'examples']),
    install_requires=requirements,
    package_data={'voltron': ['config/*']},
    entry_points={
        'console_scripts': ['voltron=voltron:main'],
        'pygments.lexers': [
            'lldb_intel = voltron.lexers:LLDBIntelLexer',
            'lldb_att = voltron.lexers:LLDBATTLexer',
            'gdb_intel = voltron.lexers:GDBIntelLexer',
            'gdb_att = voltron.lexers:GDBATTLexer',
            'vdb_intel = voltron.lexers:VDBIntelLexer',
            'vdb_att = voltron.lexers:VDBATTLexer',
            'windbg_intel = voltron.lexers:WinDbgIntelLexer',
            'windbg_att = voltron.lexers:WinDbgATTLexer',
            'capstone_intel = voltron.lexers:CapstoneIntelLexer',
        ],
        'pygments.styles': [
            'volarized = voltron.styles:VolarizedStyle',
        ]
    },
    zip_safe=False
)



import logging

from voltron.view import *
from voltron.plugin import *
from voltron.api import *

log = logging.getLogger('view')


class BacktraceView (TerminalView):
    def build_requests(self):
        return [api_request('command', block=self.block, command='bt')]

    def render(self, results):
        [res] = results

        # Set up header and error message if applicable
        self.title = '[backtrace]'

        # don't render if it timed out, probably haven't stepped the debugger again
        if res.timed_out:
            return

        if res and res.is_success:
            # Get the command output
            self.body = res.output
        else:
            log.error("Error getting backtrace: {}".format(res.message))
            self.body = self.colour(res.message, 'red')

        # Call parent's render method
        super(BacktraceView, self).render(results)


class BacktraceViewPlugin(ViewPlugin):
    plugin_type = 'view'
    name = 'backtrace'
    aliases = ('t', 'bt', 'back')
    view_class = BacktraceView



import os
import logging
import socket
import select
import threading
import logging
import logging.config
import json
import inspect
import base64
import six

from collections import defaultdict

from scruffy.plugin import Plugin

import voltron
from .plugin import APIPlugin

log = logging.getLogger('api')

version = 1.1


class InvalidRequestTypeException(Exception):
    """
    Exception raised when the client is requested to send an invalid request type.
    """
    pass


class InvalidDebuggerHostException(Exception):
    """
    Exception raised when the debugger host is invalid.
    """
    pass


class InvalidViewNameException(Exception):
    """
    Exception raised when an invalid view name is specified.
    """
    pass


class InvalidMessageException(Exception):
    """
    Exception raised when an invalid API message is received.
    """
    pass


class ServerSideOnlyException(Exception):
    """
    Exception raised when a server-side method is called on an APIMessage
    subclass that exists on the client-side.

    See @server_side decorator.
    """
    pass


class ClientSideOnlyException(Exception):
    """
    Exception raised when a server-side method is called on an APIMessage
    subclass that exists on the client-side.

    See @client_side decorator.
    """
    pass


class DebuggerNotPresentException(Exception):
    """
    Raised when an APIRequest is dispatched without a valid debugger present.
    """
    pass


class NoSuchTargetException(Exception):
    """
    Raised when an APIRequest specifies an invalid target.
    """
    pass


class TargetBusyException(Exception):
    """
    Raised when an APIRequest specifies a target that is currently busy and
    cannot be queried.
    """
    pass


class MissingFieldError(Exception):
    """
    Raised when an APIMessage is validated and has a required field missing.
    """
    pass


class NoSuchThreadException(Exception):
    """
    Raised when the specified thread ID or index does not exist.
    """
    pass


class UnknownArchitectureException(Exception):
    """
    Raised when the debugger host is running in an unknown architecture.
    """
    pass


class BlockingNotSupportedError(Exception):
    """
    Raised when a view that does not support blocking connects to a debugger
    host that does not support async mode.
    """
    pass


def server_side(func):
    """
    Decorator to designate an API method applicable only to server-side
    instances.

    This allows us to use the same APIRequest and APIResponse subclasses on the
    client and server sides without too much confusion.
    """
    def inner(*args, **kwargs):
        if args and hasattr(args[0], 'is_server') and not voltron.debugger:
            raise ServerSideOnlyException("This method can only be called on a server-side instance")
        return func(*args, **kwargs)
    return inner


def client_side(func):
    """
    Decorator to designate an API method applicable only to client-side
    instances.

    This allows us to use the same APIRequest and APIResponse subclasses on the
    client and server sides without too much confusion.
    """
    def inner(*args, **kwargs):
        if args and hasattr(args[0], 'is_server') and voltron.debugger:
            raise ClientSideOnlyException("This method can only be called on a client-side instance")
        return func(*args, **kwargs)
    return inner


def cast_b(val):
    if isinstance(val, six.binary_type):
        return val
    elif isinstance(val, six.text_type):
        return val.encode('latin1')
    return six.binary_type(val)


def cast_s(val):
    if type(val) == six.text_type:
        return val
    elif type(val) == six.binary_type:
        return val.decode('latin1')
    return six.text_type(val)


class APIMessage(object):
    """
    Top-level API message class.
    """
    _top_fields = ['type']
    _fields = {}
    _encode_fields = []

    type = None

    def __init__(self, data=None, *args, **kwargs):
        # process any data that was passed in
        if data:
            self.from_json(data)

        # any other kwargs are treated as field values
        for field in kwargs:
            setattr(self, field, kwargs[field])

    def __str__(self):
        """
        Return a string (JSON) representation of the API message properties.
        """
        return self.to_json()

    def to_dict(self):
        """
        Return a transmission-safe dictionary representation of the API message properties.
        """
        d = {field: getattr(self, field) for field in self._top_fields if hasattr(self, field)}

        # set values of data fields
        d['data'] = {}
        for field in self._fields:
            if hasattr(self, field):
                # base64 encode the field for transmission if necessary
                if field in self._encode_fields:
                    val = getattr(self, field)
                    if val:
                        val = cast_s(base64.b64encode(cast_b(val)))
                    d['data'][field] = val
                else:
                    d['data'][field] = getattr(self, field)

        return d

    def from_dict(self, d):
        """
        Initialise an API message from a transmission-safe dictionary.
        """
        for key in d:
            if key == 'data':
                for dkey in d['data']:
                    if dkey in self._encode_fields:
                        setattr(self, str(dkey), base64.b64decode(d['data'][dkey]))
                    else:
                        setattr(self, str(dkey), d['data'][dkey])
            else:
                setattr(self, str(key), d[key])

    def to_json(self):
        """
        Return a JSON representation of the API message properties.
        """
        return json.dumps(self.to_dict())

    def from_json(self, data):
        """
        Initialise an API message from a JSON representation.
        """
        try:
            d = json.loads(data)
        except ValueError:
            raise InvalidMessageException()
        self.from_dict(d)

    def __getattr__(self, name):
        """
        Attribute accessor.

        If a defined field is requested that doesn't have a value set,
        return None.
        """
        if name in self._fields:
            return None

    def validate(self):
        """
        Validate the message.

        Ensure all the required fields are present and not None.
        """
        required_fields = list(filter(lambda x: self._fields[x], self._fields.keys()))
        for field in (self._top_fields + required_fields):
            if not hasattr(self, field) or hasattr(self, field) and getattr(self, field) == None:
                raise MissingFieldError(field)


class APIRequest(APIMessage):
    """
    An API request object. Contains functions and accessors common to all API
    request types.

    Subclasses of APIRequest are used on both the client and server sides. On
    the server side they are instantiated by Server's `handle_request()`
    method. On the client side they are instantiated by whatever class is doing
    the requesting (probably a view class).
    """
    _top_fields = ['type', 'request', 'block', 'timeout']
    _fields = {}

    type = 'request'
    request = None
    block = False
    timeout = 10

    response = None
    wait_event = None
    timed_out = False

    @server_side
    def dispatch(self):
        """
        In concrete subclasses this method will actually dispatch the request
        to the debugger host and return a response. In this case it raises an
        exception.
        """
        raise NotImplementedError("Subclass APIRequest")

    @server_side
    def wait(self):
        """
        Wait for the request to be dispatched.
        """
        self.wait_event = threading.Event()
        timeout = int(self.timeout) if self.timeout else None
        self.timed_out = not self.wait_event.wait(timeout)

    def signal(self):
        """
        Signal that the request has been dispatched and can return.
        """
        self.wait_event.set()


class APIBlockingRequest(APIRequest):
    """
    An API request that blocks by default.
    """
    block = True


class APIResponse(APIMessage):
    """
    An API response object. Contains functions and accessors common to all API
    response types.

    Subclasses of APIResponse are used on both the client and server sides. On
    the server side they are instantiated by the APIRequest's `dispatch` method
    in order to serialise and send to the client. On the client side they are
    instantiated by the Client class and returned by `send_request`.
    """
    _top_fields = ['type', 'status']
    _fields = {}

    type = 'response'
    status = None

    @property
    def is_success(self):
        return self.status == 'success'

    @property
    def is_error(self):
        return self.status == 'error'

    def __repr__(self):
        return "<%s: success = %s, error = %s, body: %s>" % (
                str(self.__class__),
                self.is_success,
                self.is_error,
                {f: getattr(self, f) for f in self._top_fields + list(self._fields.keys())}
        )


class APISuccessResponse(APIResponse):
    """
    A generic API success response.
    """
    status = 'success'


class APIErrorResponse(APIResponse):
    """
    A generic API error response.
    """
    _fields = {'code': True, 'message': True}

    status = 'error'

    @property
    def timed_out(self):
        return self.code == APITimedOutErrorResponse.code


class APIGenericErrorResponse(APIErrorResponse):
    code = 0x1000
    message = "An error occurred"

    def __init__(self, message=None):
        super(APIGenericErrorResponse, self).__init__()
        if message:
            self.message = message


class APIInvalidRequestErrorResponse(APIErrorResponse):
    code = 0x1001
    message = "Invalid API request"


class APIPluginNotFoundErrorResponse(APIErrorResponse):
    code = 0x1002
    message = "Plugin was not found for request"


class APIDebuggerHostNotSupportedErrorResponse(APIErrorResponse):
    code = 0x1003
    message = "The targeted debugger host is not supported by this plugin"


class APITimedOutErrorResponse(APIErrorResponse):
    code = 0x1004
    message = "The request timed out"


class APIDebuggerNotPresentErrorResponse(APIErrorResponse):
    code = 0x1004
    message = "No debugger host was found"


class APINoSuchTargetErrorResponse(APIErrorResponse):
    code = 0x1005
    message = "No such target"


class APITargetBusyErrorResponse(APIErrorResponse):
    code = 0x1006
    message = "Target busy"


class APIMissingFieldErrorResponse(APIGenericErrorResponse):
    code = 0x1007
    message = "Missing field"


class APIEmptyResponseErrorResponse(APIGenericErrorResponse):
    code = 0x1008
    message = "Empty response"


class APIServerNotRunningErrorResponse(APIGenericErrorResponse):
    code = 0x1009
    message = "Server is not running"



from voltron.view import TerminalView, VoltronView, log
from voltron.plugin import api_request, ViewPlugin
from voltron.lexers import get_lexer_by_name
import pygments
import pygments.formatters


class DisasmView(TerminalView):
    @classmethod
    def configure_subparser(cls, subparsers):
        sp = subparsers.add_parser('disasm', help='disassembly view', aliases=('d', 'dis'))
        VoltronView.add_generic_arguments(sp)
        sp.set_defaults(func=DisasmView)
        sp.add_argument('--use-capstone', '-c', action='store_true', default=False, help='use capstone')
        sp.add_argument('--address', '-a', action='store', default=None,
                        help='address (in hex or decimal) from which to start disassembly')

    def build_requests(self):
        if self.args.address:
            if self.args.address.startswith('0x'):
                addr = int(self.args.address, 16)
            else:
                try:
                    addr = int(self.args.address, 10)
                except:
                    addr = int(self.args.address, 16)
        else:
            addr = None
        req = api_request('disassemble', block=self.block, use_capstone=self.args.use_capstone,
                          offset=self.scroll_offset, address=addr)
        req.count = self.body_height()
        return [req]

    def render(self, results):
        [res] = results

        # Set up header & error message if applicable
        self.title = '[disassembly]'

        # don't render if it timed out, probably haven't stepped the debugger again
        if res.timed_out:
            return

        if res and res.is_success:
            # Get the disasm
            disasm = res.disassembly
            disasm = '\n'.join(disasm.split('\n')[:self.body_height()])

            # Highlight output
            try:
                host = 'capstone' if self.args.use_capstone else res.host
                lexer = get_lexer_by_name('{}_{}'.format(host, res.flavor))
                formatter = pygments.formatters.get_formatter_by_name(
                    self.config.format.pygments_formatter,
                    style=self.config.format.pygments_style)
                disasm = pygments.highlight(disasm, lexer, formatter)
            except Exception as e:
                log.warning('Failed to highlight disasm: ' + str(e))
                log.info(self.config.format)

            # Build output
            self.body = disasm.rstrip()
        else:
            log.error("Error disassembling: {}".format(res.message))
            self.body = self.colour(res.message, 'red')

        # Call parent's render method
        super(DisasmView, self).render(results)


class DisasmViewPlugin(ViewPlugin):
    plugin_type = 'view'
    name = 'disassembly'
    aliases = ('d', 'dis', 'disasm')
    view_class = DisasmView
